do

do
local _ENV = _ENV
package.preload[ "IO_Lua" ] = function( ... ) local arg = _G.arg;
local IO_Lua = {}

function IO_Lua.getLine ()
    return io.read ()
end

function IO_Lua.print (str)
    return print (str)
end

return IO_Lua
end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.array" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Array constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array

local Array

local arrayProto = require("castl.protos").arrayProto
local coreObjects = require("castl.core_objects")

local getmetatable, type = getmetatable, type
local pack = table.pack or function(...) return {n = select('#',...),...} end

_ENV = nil

Array = function (this, ...)
    local args = pack(...)
    local newArray = {}

    if args.n == 1 and type(args[1]) == "number" then
        newArray.length = args[1]
    else
        newArray.length = args.n
        for i = 0, args.n do
            newArray[i] = args[i + 1]
        end
    end

    return coreObjects.array(newArray, newArray.length)
end

Array.length = 1

Array.isArray = function (this, arr)
    return (getmetatable(arr) or {})._prototype == arrayProto
end

Array.prototype = arrayProto
arrayProto.constructor = Array

return Array

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.boolean" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Boolean constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean


local objectToString = require("castl.core_objects").objectToString
local booleanProto = require("castl.protos").booleanProto
local internal = require("castl.internal")

local Boolean

local setmetatable = setmetatable
local null, get, put, withinNew, ToNumber = internal.null, internal.get, internal.put, internal.withinNew, internal.ToNumber

_ENV = nil

local booleanPrimitive = function(var)
    if var == 0 or var == "" or var == null or var ~= var then
        return false
    elseif var then
        return true
    else
        return false
    end
end

Boolean = function(this, arg)
    -- Boolean constructor not called within a new
    if not withinNew(this, booleanProto) then
        return booleanPrimitive(arg)
    end

    local o = {}

    setmetatable(o, {
        __index = function (self, key)
            return get(self, booleanProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return objectToString(self)
        end,
        _primitive = booleanPrimitive(arg),
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = booleanProto
    })

    return o
end

Boolean.prototype = booleanProto
booleanProto.constructor = Boolean

return Boolean

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.date" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Date constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean

local Date

local internal = require("castl.internal")
local dateparser = require("castl.modules.dateparser")
local dateProto = require("castl.protos").dateProto

local luajit = jit ~= nil
local date, time = os.date, os.time
local pack = table.pack or function(...) return {n = select('#',...),...} end
local type, setmetatable, require, tonumber = type, setmetatable, require, tonumber
local get, put, withinNew, ToNumber = internal.get, internal.put, internal.withinNew, internal.ToNumber

_ENV = nil

Date = function(this, ...)
    -- Date constructor not called within a new
    if not withinNew(this, dateProto) then
        return date("%a %h %d %Y %H:%M:%S GMT%z (%Z)")
    end

    local args = pack(...)
    local timestamp = 0

    if args.n == 0 then
        timestamp = Date.now()
    elseif args.n == 1 then
        local arg = args[1]
        local targ = type(arg)
        if targ == "number" then
            timestamp = arg * 1000
        elseif targ == "string" then
            timestamp = Date.parse(arg)
        end
    else
        -- special behavior for year between 0 and 100
        if args[1] >= 0 and args[1] <100 then
            args[1] = 1900 + args[1]
        end
        -- more than 1 arguments
        -- year, month, day, hour, minute, second, millisecond
        timestamp = time{year=args[1],
            month=args[2] + 1,
            day=args[3] or 1,
            hour=args[4] or 0,
            min = args[5] or 0,
            sec = args[6] or 0}

        timestamp = timestamp * 1000 + (args[7] or 0)
    end

    local o = {}
    o._timestamp = timestamp

    setmetatable(o, {
        __index = function (self, key)
            return get(self, dateProto, key)
        end,
        __newindex = put,
        __tostring = dateProto.toString,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = dateProto
    })

    return o
end

Date._timestamp = 0

if luajit then
    local ffi = require("ffi")
    -- posix systems only
    ffi.cdef[[
        typedef struct timeval {
          long tv_sec;
          long tv_usec;
        } timeval;
        int gettimeofday(struct timeval *restrict tp, void *restrict tzp);
    ]]

    local te = ffi.new("timeval[1]")

    Date.now = function(this)
        ffi.C.gettimeofday(te, nil);
        return tonumber(te[0].tv_sec * 1000 + te[0].tv_usec / 1000);
    end
else
    Date.now = function(this)
        -- TODO: write a C function to get milliseconds
        return time() * 1000
    end
end

Date.parse = function(this, str)
    -- TODO: parse RFC2822 only for now
    return dateparser.parse(str, 'RFC2822') * 1000
end

Date.UTC = function(this, year, month, day, hrs, min, sec, ms)
    local timestamp = time{year=year,
        month = month + 1,
        day = day or 1,
        hour = hrs or 0,
        min = min or 0,
        sec = sec or 0}

    timestamp = (timestamp + dateProto.getTimezoneOffset()) * 1000 + (ms or 0)

    return timestamp
end

Date.length = 7

Date.prototype = dateProto
dateProto.constructor = Date

return Date

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.call_site" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL CallSite constructor submodule]] --

local CallSite

local callSiteProto = require("castl.protos").callSiteProto

_ENV = nil

CallSite = function(this, receiver, fun, pos, strict_mode)
    this.receiver = receiver
    this.fun = fun
    this.pos = pos
end

CallSite.prototype = callSiteProto
callSiteProto.constructor = CallSite

return CallSite

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Error constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Error

local Error

local errorProto = require("castl.protos").errorProto
local CallSite = require("castl.constructor.error.call_site")
local internal = require("castl.internal")
local coreObjects = require("castl.core_objects")

local new, array, instanceof = coreObjects.new, coreObjects.array, coreObjects.instanceof

local setmetatable, getmetatable, rawget, rawset = setmetatable, getmetatable, rawget, rawset
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber
local getinfo = debug.getinfo
local ipairs, remove = ipairs, table.remove

_ENV = nil

Error = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, errorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = errorProto}
    )

    Error:captureStackTrace(o, Error)

    return o
end

Error.captureStackTrace = function(this, error, constructorOpt)
    local frames = {}
    local frame_idx = 1

    local info_idx = constructorOpt ~= nil and 3 or 2
    while true do
        local frame

        if frame_idx <= Error.stackTraceLimit then
            frame = getinfo(info_idx)
        end
        if frame == nil then
            break
        end

        -- create a CallSite
        local cs = new(CallSite, nil, frame.fun, 0)
        getmetatable(cs).frame = frame
        frames[frame_idx] = cs
        frame_idx = frame_idx + 1
        info_idx = info_idx + 1
    end

    -- add marker
    getmetatable(error).__stack = true

    error._gstack = function(this)
        if Error.prepareStackTrace then
            -- shift to 0-based index
            local tmp = rawget(frames, 1)
            remove(frames, 1)
            rawset(frames, 0, tmp)
            return Error:prepareStackTrace(this, array(frames, frame_idx - 1))
        end

        local s = instanceof(error, Error) and error:toString() or "Error"
        for _, frame in ipairs(frames) do
            s = s .. "\n    at " .. frame:toString()
        end
        return s
    end
end

Error.stackTraceLimit = 10

Error.prototype = errorProto
errorProto.constructor = Error

return Error

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.eval_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL EvalError constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/EvalError

local EvalError

local Error = require("castl.constructor.error.error")
local evalErrorProto = require("castl.protos").evalErrorProto
local internal = require("castl.internal")

local setmetatable = setmetatable
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber

_ENV = nil

EvalError = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, evalErrorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = evalErrorProto})

    Error:captureStackTrace(o, EvalError)

    return o
end

EvalError.prototype = evalErrorProto
evalErrorProto.constructor = EvalError

return EvalError

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.range_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL RangeError constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RangeError

local RangeError

local Error = require("castl.constructor.error.error")
local rangeErrorProto = require("castl.protos").rangeErrorProto
local internal = require("castl.internal")

local setmetatable = setmetatable
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber

_ENV = nil

RangeError = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, rangeErrorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = rangeErrorProto})

    Error:captureStackTrace(o, RangeError)

    return o
end

RangeError.prototype = rangeErrorProto
rangeErrorProto.constructor = RangeError

return RangeError

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.reference_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL ReferenceError constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/ReferenceError

local ReferenceError

local Error = require("castl.constructor.error.error")
local referenceErrorProto = require("castl.protos").referenceErrorProto
local internal = require("castl.internal")

local setmetatable = setmetatable
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber

_ENV = nil

ReferenceError = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, referenceErrorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = referenceErrorProto})

    Error:captureStackTrace(o, ReferenceError)

    return o
end

ReferenceError.prototype = referenceErrorProto
referenceErrorProto.constructor = ReferenceError

return ReferenceError

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.syntax_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL SyntaxError constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/SyntaxError

local SyntaxError

local Error = require("castl.constructor.error.error")
local syntaxErrorProto = require("castl.protos").syntaxErrorProto
local internal = require("castl.internal")

local setmetatable = setmetatable
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber

_ENV = nil

SyntaxError = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, syntaxErrorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = syntaxErrorProto})

    Error:captureStackTrace(o, SyntaxError)

    return o
end

SyntaxError.prototype = syntaxErrorProto
syntaxErrorProto.constructor = SyntaxError

return SyntaxError

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.type_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL TypeError constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/TypeError

local TypeError

local Error = require("castl.constructor.error.error")
local typeErrorProto = require("castl.protos").typeErrorProto
local internal = require("castl.internal")

local setmetatable = setmetatable
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber

_ENV = nil

TypeError = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, typeErrorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = typeErrorProto})

    Error:captureStackTrace(o, TypeError)

    return o
end

TypeError.prototype = typeErrorProto
typeErrorProto.constructor = TypeError

return TypeError

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.error.uri_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL URIError constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/URIError

local URIError

local Error = require("castl.constructor.error.error")
local uriErrorProto = require("castl.protos").uriErrorProto
local internal = require("castl.internal")

local setmetatable = setmetatable
local get, put, ToNumber = internal.get, internal.put, internal.ToNumber

_ENV = nil

URIError = function(this, message)
    local o = {}
    o.message = message

    setmetatable(o, {
        __index = function (self, key)
            return get(self, uriErrorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = uriErrorProto})

    Error:captureStackTrace(o, URIError)

    return o
end

URIError.prototype = uriErrorProto
uriErrorProto.constructor = URIError

return URIError

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.function" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Function constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function

local esprima, castl, runtime
local luajit = jit ~= nil

local errorHelper = require("castl.modules.error_helper")
local throw = require("castl.jssupport").throw
local functionProto = require("castl.protos").functionProto

local Function

local pack = table.pack or function(...) return {n = select('#',...),...} end
local require, assert, load, tinsert, concat = require, assert, load, table.insert, table.concat

_ENV = nil

Function = function(this, ...)
    if ... then
        local args = pack(...)
        local body = args[args.n]

        if luajit then
            castl = castl or require("castl.jscompile.castl_jit")
            esprima = esprima or require("castl.jscompile.esprima_jit")
        else
            castl = castl or require("castl.jscompile.castl")
            esprima = esprima or require("castl.jscompile.esprima")
        end

        -- parse body of the function in error-tolerant mode
        local esprimaOptions = {tolerant = true}
        local ast = esprima:parse(body, esprimaOptions)

        -- get the params
        local params = {"local this"}
        for i = 1, args.n - 1 do
            tinsert(params, ",")
            tinsert(params, args[i])
        end
        tinsert(params, " = ...;\n")

        -- compile the ast
        -- castl used in eval mode
        local castlOptions = {jit = luajit}
        local castlResult = castl:compileAST(ast, castlOptions)

        local compiledFunction
        if castlResult.success then
            local luaCode = concat(params) .. castlResult.compiled
            -- eval lua code
            runtime = runtime or require("castl.runtime")
            compiledFunction = assert(load(luaCode, nil, "t", runtime))
        else
            throw(errorHelper.newEvalError("Function(): Failed to compile AST to Lua code"))
        end

        return compiledFunction
    end

    return function () end
end

Function.prototype = functionProto
functionProto.constructor = Function

return Function

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.number" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Number constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number

local Number

local coreObjects = require("castl.core_objects")
local internal = require("castl.internal")
local numberProto = require("castl.protos").numberProto
local jssupport = require("castl.jssupport")
local globalFunctions = require("castl.global_functions")

local huge = math.huge
local type, setmetatable = type, setmetatable
local get, put, withinNew, ToNumber = internal.get, internal.put, internal.withinNew, internal.ToNumber

_ENV = nil

Number = function(this, arg)
    arg = ToNumber(arg)
    -- Number constructor not called within a new
    if not withinNew(this, numberProto) then
        return arg
    end

    local o = {}

    setmetatable(o, {
        __index = function (self, key)
            return get(self, numberProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return coreObjects.objectToString(self)
        end,
        _primitive = arg,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = numberProto
    })

    return o
end

Number.isFinite = function(this, arg)
    if type(arg) == 'number' then
        return arg ~= huge and arg ~= -huge and not (arg ~= arg)
    end

    return false
end

Number.isNaN = function (this, arg)
    if type(arg) == 'number' then
        return (arg ~= arg)
    end

    return false
end

Number.parseInt = function (this, str, radix)
    return globalFunctions.parseInt(this, str, radix)
end

Number.parseFloat = function (this, str)
    return globalFunctions.parseFloat(this, str)
end

-- Static properties
Number.NaN = 0/0
Number.MAX_VALUE = 1.7976931348623157e+308
Number.MIN_VALUE = 5e-324
Number.NEGATIVE_INFINITY = -jssupport.Infinity
Number.POSITIVE_INFINITY = jssupport.Infinity

Number.prototype = numberProto
numberProto.constructor = Number

return Number

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.object" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Object constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object

local Object

local coreObjects = require("castl.core_objects")
local internal = require("castl.internal")
local objectProto = require("castl.protos").objectProto
local errorHelper = require("castl.modules.error_helper")
local throw = require("castl.jssupport").throw

local type, pairs, ipairs, tostring = type, pairs, ipairs, tostring
local getmetatable, rawset, rawget = getmetatable, rawset, rawget
local null, setNewMetatable, ToObject, ToString = internal.null, internal.setNewMetatable, internal.ToObject, internal.ToString
local getFunctionProxy = internal.getFunctionProxy

_ENV = nil

Object = function (this, obj)
    if obj == nil or obj == null then
        return coreObjects.obj({})
    end

    return ToObject(obj)
end

Object.create = function (this, prototype, props)
    -- get function proxy
    if type(prototype) == "function" then
        prototype = getFunctionProxy(prototype)
    end
    if type(prototype) ~= 'table' then
        throw(errorHelper.newTypeError("Object prototype may only be an Object or null"))
    end

    local o = {}

    if prototype ~= null then
        setNewMetatable(o, prototype)
    end

    if props ~= nil then
        Object:defineProperties(o, props)
    end

    return o
end

Object.defineProperties = function (this, obj, props)
    if props ~= nil then
        for k, v in pairs(props) do
            Object:defineProperty(obj, k, v)
        end
    end
    return obj
end

Object.defineProperty = function(this, obj, prop, descriptor)
    local isFunction = type(obj) == 'function'
    local originalObj = obj
    -- get function proxy
    if isFunction then
        obj = getFunctionProxy(obj)
    end

    if type(obj) ~= 'table' or obj == null then
        throw(errorHelper.newTypeError("Object.defineProperty called on non-object"))
    end
    if type(descriptor) ~= 'table' then
        throw(errorHelper.newTypeError("Property description must be an object: " .. tostring(descriptor)))
    end
    if descriptor.value ~= nil and (descriptor.get ~= nil or descriptor.set ~= nil) then
        throw(errorHelper.newTypeError("Invalid property.  A property cannot both have accessors and be writable or have a value"))
    end

    -- value
    if descriptor.value ~= nil then
        -- TODO: related to weak typing
        if type(prop) ~= "number" then
            prop = ToString(prop)
        end
        rawset(obj, prop, descriptor.value)

        if isFunction then
            return originalObj
        end
        return obj
    end

    prop = ToString(prop)

    -- getter
    if descriptor.get ~= nil then
        rawset(obj, "_g" .. prop, descriptor.get)
    end

    -- setter
    if descriptor.set ~= nil then
        rawset(obj, "_s" .. prop, descriptor.set)
    end

    if isFunction then
        return originalObj
    end

    return obj
end

-- TODO
Object.freeze = function (this, obj)
    -- not really frozen...
    return obj
end

Object.getOwnPropertyDescriptor = function (this, obj, prop)
    -- get function proxy
    if type(obj) == 'function' then
        obj = getFunctionProxy(obj)
    end
    if type(obj) ~= 'table' or obj == null then
        throw(errorHelper.newTypeError("Object.getOwnPropertyDescriptor called on non-object"))
    end

    local value = rawget(obj, prop)

    if value ~= nil then
        return coreObjects.obj({
            value = value,
            writable = true,
            configurable = true,
            enumerable = true
        })
    end

    return nil
end

Object.getOwnPropertyNames = function (this, obj)
    -- get function proxy
    if type(obj) == 'function' then
        obj = getFunctionProxy(obj)
    end
    if type(obj) ~= 'table' or obj == null then
        throw(errorHelper.newTypeError("Object.getOwnPropertyNames called on non-object"))
    end

    local ret, i = {}, 0

    local props = coreObjects.props(obj, false, true)
    for _, key in ipairs(props) do
        ret[i] = tostring(key)
        i = i + 1
    end

    return coreObjects.array(ret, i)
end

Object.getPrototypeOf = function (this, obj)
    if type(obj) == 'function' then
        -- return functionProto
        return getmetatable(obj)._prototype
    end
    if type(obj) ~= 'table' or obj == null then
        throw(errorHelper.newTypeError("Object.getPrototypeOf called on non-object"))
    end

    local mt = getmetatable(obj)
    if mt then
        return mt._prototype
    end

    return nil
end

-- TODO
Object.isExtensible = function (this, obj)
    return true
end

-- TODO
Object.isFrozen = function (this, obj)
    return false
end

-- TODO
Object.isSealed = function (this, obj)
    return false
end

Object.keys = function (this, obj)
    -- get function proxy
    if type(obj) == 'function' then
        obj = getFunctionProxy(obj)
    end

    if type(obj) ~= 'table' or obj == null then
        throw(errorHelper.newTypeError("Object.keys called on non-object"))
    end

    local ret, i = {}, 0

    local props = coreObjects.props(obj, false, false)
    for _, key in ipairs(props) do
        ret[i] = tostring(key)
        i = i + 1
    end

    return coreObjects.array(ret, i)
end

-- TODO
Object.preventExtensions = function (this, obj)
    return obj
end

-- TODO
Object.seal = function (this, obj)
    return obj
end

Object.prototype = objectProto
objectProto.constructor = Object

return Object

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.regexp" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL RegExp constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RegExp

local RegExp

local regexpProto = require("castl.protos").regexpProto
local internal = require("castl.internal")
local errorHelper = require("castl.modules.error_helper")
local throw = require("castl.jssupport").throw

local find = string.find
local setmetatable, getmetatable = setmetatable, getmetatable
local ToString, get, put, withinNew, ToNumber = internal.ToString, internal.get, internal.put, internal.withinNew, internal.ToNumber

_ENV = nil

RegExp = function(this, pattern, flags)
    local patternIsRegexp = (getmetatable(pattern) or {})._prototype == regexpProto

    if patternIsRegexp and flags ~= nil then
        throw(errorHelper.newTypeError("Cannot supply flags when constructing one RegExp from another"))
    end

    if not withinNew(this, regexpProto) and patternIsRegexp then
        return pattern
    end

    flags = flags or ""

    local o = {}

    if patternIsRegexp then
        o.source = ToString(pattern.source)
        o.global = pattern.global
        o.ignoreCase = pattern.ignoreCase
        o.multiline = pattern.multiline
    else
        o.source = ToString(pattern)
        o.global = find(flags, "g") and true
        o.ignoreCase = find(flags, "i") and true
        o.multiline = find(flags, "m") and true
    end

    setmetatable(o, {
        __index = function (self, key)
            return get(self, regexpProto, key)
        end,
        __newindex = put,
        __tostring = regexpProto.toString,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = regexpProto
    })

    return o

end

RegExp.prototype = regexpProto
regexpProto.constructor = RegExp

return RegExp

end
end

do
local _ENV = _ENV
package.preload[ "castl.constructor.string" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL String constructor submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String

local String

local stringProto = require("castl.protos").stringProto
local bit = require("castl.modules.bit")
local coreObjects = require("castl.core_objects")
local internal = require("castl.internal")

local bor, band, arshift = bit.bor, bit.band, bit.arshift
local pack = table.pack or function(...) return {n = select('#',...),...} end
local unpack, tinsert, concat, stochar =  table.unpack or unpack, table.insert, table.concat, string.char
local setmetatable = setmetatable
local ToString, withinNew, get, put, ToNumber, ToUint16 = internal.ToString, internal.withinNew, internal.get, internal.put, internal.ToNumber, internal.ToUint16

_ENV = nil

String = function(this, ...)
    local arg = pack(...).n > 0 and ToString(...) or "";

    -- String constructor not called within a new
    if not withinNew(this, stringProto) then
        return arg
    end

    local o = {length = arg.length}

    for i = 0, arg.length - 1 do
        o[i] = arg[i]
    end

    setmetatable(o, {
        __index = function (self, key)
            return get(self, stringProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return coreObjects.objectToString(self)
        end,
        _primitive = arg,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = stringProto
    })

    return o
end

local toUTF8Array = function (charcode)
    local utf8 = {}

    if charcode < 128 then
        tinsert(utf8, charcode)
    elseif charcode < 2048 then
        tinsert(utf8, bor(192, arshift(charcode, 6)))
        tinsert(utf8, bor(128, band(charcode, 63)))
    elseif (charcode < 55296) or (charcode >= 57344) then
        tinsert(utf8, bor(224, arshift(charcode, 12)))
        tinsert(utf8, bor(128, band(arshift(charcode, 6), 63)))
        tinsert(utf8, bor(128, band(charcode, 63)))
    end

    return utf8
end

String.fromCharCode = function(this, ...)
    local args = pack(...)
    local str = {}
    for i = 1, args.n do
        local charCode = ToUint16(args[i]);
        local char = stochar(unpack(toUTF8Array(charCode)))

        tinsert(str, char)
    end

    return concat(str)
end

String.prototype = stringProto
stringProto.constructor = String

return String

end
end

do
local _ENV = _ENV
package.preload[ "castl.core_objects" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL JS core objects submodule]] --

local internal = require("castl.internal")
local errorHelper = require("castl.modules.error_helper")
local throw = require("castl.jssupport").throw
local typeof = require("castl.jssupport").typeof
local RegExp

local protos = require("castl.protos")

local objectProto = protos.objectProto
local functionProto = protos.functionProto
local arrayProto = protos.arrayProto
local booleanProto = protos.booleanProto
local numberProto = protos.numberProto
local stringProto = protos.stringProto
local dateProto = protos.dateProto
local regexpProto = protos.regexpProto
local errorProto = protos.errorProto
local rangeErrorProto = protos.rangeErrorProto
local referenceErrorProto = protos.referenceErrorProto
local syntaxErrorProto = protos.syntaxErrorProto
local typeErrorProto = protos.typeErrorProto
local uriErrorProto = protos.uriErrorProto
local evalErrorProto = protos.evalErrorProto

local coreObjects = {}

-- Dependencies
local getmetatable, setmetatable, rawget, rawset = getmetatable, setmetatable, rawget, rawset
local debug = debug
local type, max, strlen, strsub, tonumber = type, math.max, string.len, string.sub, tonumber
local pack = table.pack or function(...) return {n = select('#',...),...} end
local tinsert, concat, sort = table.insert, table.concat, table.sort
local pairs, ipairs, tostring = pairs, ipairs, tostring
local require = require
local getPrototype, get, put  = internal.prototype, internal.get, internal.put
local setNewMetatable, ToNumber, getFunctionProxy = internal.setNewMetatable, internal.ToNumber, internal.getFunctionProxy

_ENV = nil

-- Core objects metatables
local objectMt, arrayMt = {}, {}
local booleanMt, numberMt, stringMt = {}, {}, {}
local functionMt, undefinedMt = {}, {}

-- Hidden field _prototype keep reference to inherited table
objectMt._prototype = objectProto
functionMt._prototype = functionProto
arrayMt._prototype = arrayProto

coreObjects.objectToString = function(o)
    local ret = {"{ "}
    local elements = {}

    local props = coreObjects.props(o, false, false)
    for _, key in ipairs(props) do
        local k = key
        local v = o[key]

        if type(k) == "number" then
            k = "'" .. tostring(k) .. "'"
        end

        if type(v) == "string" then
            v = "'" .. v .. "'"
        else
            v = tostring(v)
        end

        tinsert(elements, k .. ": " .. v)
    end
    tinsert(ret, concat(elements, ", "))
    tinsert(ret, " }")

    return concat(ret, "")
end

--[[
    Object metatable
--]]

objectMt.__index = function(self, key)
    return get(self, objectProto, key)
end

objectMt.__newindex = put

objectMt.__tostring = function(self)
    return coreObjects.objectToString(self)
end

objectMt.__sub = function(a, b)
    return ToNumber(a) - ToNumber(b)
end

objectMt.__mul = function(a, b)
    return ToNumber(a) * ToNumber(b)
end

objectMt.__div = function(a, b)
    return ToNumber(a) / ToNumber(b)
end

--[[
    Function metatable
--]]

functionMt.__index = function(self, key)
    local proxy = getFunctionProxy(self)

    local value = rawget(proxy, key)
    if value ~= nil then
        return value
    end

    return get(proxy, functionProto, key)
end

functionMt.__newindex = function(self, key, value)
    local proxy = getFunctionProxy(self)
    put(proxy, key, value)
end

functionMt.__tostring = function(self)
    return "[Function]"
end

functionMt.__sub = function(a, b)
    return ToNumber(a) - ToNumber(b)
end

functionMt.__mul = function(a, b)
    return ToNumber(a) * ToNumber(b)
end

functionMt.__div = function(a, b)
    return ToNumber(a) / ToNumber(b)
end

debug.setmetatable((function () end), functionMt)

--[[
    Array metatable
--]]

arrayMt.__index = function(self, key)
    return get(self, arrayProto, key)
end

arrayMt.__newindex = function(self, key, value)
    if type(key) == 'number' then
        local length = rawget(self, 'length') or 0
        rawset(self, 'length', max(length, key + 1))
    end
    put(self, key, value)
end

arrayMt.__tostring = function(self)
    local ret = {"[ "}
    local elements = {}

    local props = coreObjects.props(self, false, false)
    for _, key in ipairs(props) do
        local k = key
        local v = self[key]

        if type(v) == "string" then
            v = "'" .. v .. "'"
        else
            v = tostring(v)
        end

        if type(k) == "number" then
            tinsert(elements, v)
        else
            tinsert(elements, k .. ": " .. v)
        end
    end
    tinsert(ret, concat(elements, ", "))
    tinsert(ret, " ]")

    return concat(ret, "")
end

arrayMt.__sub = function(a, b)
    return ToNumber(a) - ToNumber(b)
end

arrayMt.__mul = function(a, b)
    return ToNumber(a) * ToNumber(b)
end

arrayMt.__div = function(a, b)
    return ToNumber(a) / ToNumber(b)
end

--[[
    Boolean metatable
--]]

booleanMt.__index = function(self, key)
    return get(nil, booleanProto, key)
end

-- immutable
booleanMt.__newindex = function() end

booleanMt.__sub = function(a, b)
    return ToNumber(a) - ToNumber(b)
end

booleanMt.__mul = function(a, b)
    return ToNumber(a) * ToNumber(b)
end

booleanMt.__div = function(a, b)
    return ToNumber(a) / ToNumber(b)
end

debug.setmetatable(true, booleanMt)

--[[
    Number metatable
--]]

numberMt.__index = function(self, key)
    return get(nil, numberProto, key)
end

-- immutable
numberMt.__newindex = function() end

debug.setmetatable(0, numberMt)

--[[
    String metatable
--]]

stringMt.__index = function(self, key)
    local length = strlen(self)

    if key == "length" then
        return length
    end

    -- Access characters of a string like an array
    local num = tonumber(key)
    if num then
        if num < length then
            return strsub(self, num + 1, num + 1)
        end
        return nil
    end

    return get(nil, stringProto, key)
end

-- immutable
stringMt.__newindex = function() end

stringMt.__sub = function(a, b)
    return ToNumber(a) - ToNumber(b)
end

stringMt.__mul = function(a, b)
    return ToNumber(a) * ToNumber(b)
end

stringMt.__div = function(a, b)
    return ToNumber(a) / ToNumber(b)
end

debug.setmetatable("", stringMt)

--[[
    Nil metatable
--]]

undefinedMt.__index = function(self, key)
    throw(errorHelper.newTypeError("Cannot read property '" .. key .. "' of undefined"))
end

undefinedMt.__newindex = function(self, key)
    throw(errorHelper.newTypeError("Cannot set property '" .. key .. "' of undefined"))
end

undefinedMt.__tostring = function ()
    return "undefined"
end

undefinedMt.__sub = function ()
    return 0/0
end
undefinedMt.__mul = function ()
    return 0/0
end
undefinedMt.__div = function ()
    return 0/0
end

debug.setmetatable(nil, undefinedMt)

--[[
    Support functions
--]]

-- Inline creation of object: {att1: "...", att2: ...}
function coreObjects.obj(o)
    return setmetatable(o, objectMt)
end

-- Inline creation of array: [..., ...]
function coreObjects.array(arr, length)
    rawset(arr, 'length', length)
    return setmetatable(arr, arrayMt)
end

-- Inline creation of RegExp: /.../
function coreObjects.regexp(pattern, flags)
    RegExp = RegExp or require("castl.constructor.regexp")
    return coreObjects.new(RegExp, pattern, flags)
end

-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/new
function coreObjects.new(f, ...)
    if type(f) ~= "function" then
        throw(errorHelper.newTypeError(typeof(f) .. " is not a function"))
    end

    local o = {}
    setNewMetatable(o, f.prototype)
    local ret = f(o, ...)

    -- http://stackoverflow.com/a/3658673
    local tr = type(ret)
    if tr == "table" or tr == "function" then
        return ret
    end

    return o
end

function coreObjects.arguments(...)
    local args, obj = pack(...), {}

    -- make a 0 based numbering array like
    -- and we ignore the first argument (= this)
    for i = 2, args.n do
        obj[i - 2] = args[i]
    end

    obj.length = args.n - 1

    local mt = {
        __index = function (self, key)
            return get(self, objectProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return coreObjects.objectToString(self)
        end,
        _arguments = true,
        _prototype = objectProto
    }

    setmetatable(obj, mt)

    return obj
end

function coreObjects.instanceof(object, class)
    if type(class) ~= "function" then
        throw(errorHelper.newTypeError("Expecting a function in instanceof check, but got " .. tostring(class)))
    end

    if class.prototype then
        local classPrototypeAttribute = class.prototype
        local objectPrototype = getPrototype(object)

        while objectPrototype do
            if objectPrototype == classPrototypeAttribute then
                return true
            end
            objectPrototype = getPrototype(objectPrototype)
        end
    end
    return false
end

function coreObjects.props (arg, inherited, enumAll)
    if type(arg) == 'function' then
        arg = getFunctionProxy(arg)
    end

    local ret = {}

    local mt = getmetatable(arg)
    local isArrayLike = (mt and (mt._prototype == arrayProto or mt._arguments or mt._prototype == stringProto));

    if isArrayLike then
        return coreObjects.propsArr(arg, enumAll)
    elseif type(arg) == 'table' then
        return coreObjects.propsObj(arg, inherited)
    elseif type(arg) == 'string' then
        for i = 0, arg.length - 1 do
            tinsert(ret, i)
        end
    end

    return ret
end

function coreObjects.propsObj(arg, inherited)
    local ret = {}
    repeat
        for i in pairs(arg) do
            tinsert(ret, i)
        end
        arg = (getmetatable(arg) or {})._prototype
    until not inherited or arg == nil
        -- Hide properties of builtin-object
        or arg == objectProto or arg == numberProto
        or arg == booleanProto or arg == stringProto
        or arg == arrayProto or arg == functionProto
        or arg == dateProto or arg == regexpProto

    -- sort keys
    sort(ret, function(a, b)
        a = tonumber(a) or a
        b = tonumber(b) or b

        local ta, tb = type(a), type(b)
        if ta == "number" and tb == "string" then
            return true
        elseif ta == "string" and tb == "number" then
            return false
        end
        return a < b
    end)

    return ret
end

function coreObjects.propsArr(arg, enumAll)
    local ret = {}

    for i in pairs(arg) do
        if enumAll or not (i == "length") then
            tinsert(ret, i)
        end
    end

    -- sort keys
    sort(ret, function(a, b)
        local ta, tb = type(a), type(b)
        if ta == "number" and tb == "string" then
            return true
        elseif ta == "string" and tb == "number" then
            return false
        end
        return a < b
    end)

    return ret
end

--[[
    Prototypes inheritance
--]]

-- Object.prototype has no _prototype
setmetatable(objectProto, {
    __index = function(self, key)
        return get(self, objectProto, key)
    end,
    __newindex = put,
    __tostring = function(self)
        return coreObjects.objectToString(self)
    end,
    __sub = function(a, b)
        return ToNumber(a) - ToNumber(b)
    end,
    __mul = function(a, b)
        return ToNumber(a) * ToNumber(b)
    end,
    __div = function(a, b)
        return ToNumber(a) / ToNumber(b)
    end
})

-- inherit from Object
coreObjects.obj(functionProto)
coreObjects.obj(arrayProto)
coreObjects.obj(booleanProto)
coreObjects.obj(numberProto)
coreObjects.obj(stringProto)
coreObjects.obj(dateProto)
coreObjects.obj(regexpProto)
coreObjects.obj(errorProto)

local err = function(o)
    setmetatable(o, {
        __index = function (self, key)
            return get(self, errorProto, key)
        end,
        __newindex = put,
        __tostring = function(self)
            return self:toString()
        end,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = errorProto})
end

-- inherit from Error
err(rangeErrorProto)
err(referenceErrorProto)
err(syntaxErrorProto)
err(typeErrorProto)
err(uriErrorProto)
err(evalErrorProto)


--[[
    Export objectMt and arrayMt for JSON.parse
--]]

coreObjects.objectMt = objectMt
coreObjects.arrayMt = arrayMt

-- global this
coreObjects.this = coreObjects.obj({})

return coreObjects

end
end

do
local _ENV = _ENV
package.preload[ "castl.eval" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local castl, esprima, runtime
local luajit = jit ~= nil

local eval = {}
local errorHelper = require("castl.modules.error_helper")
local throw = require("castl.jssupport").throw

local debug, setmetatable, assert, load, require, type = debug, setmetatable, assert, load, require, type

_ENV = nil

-- http://stackoverflow.com/a/2835433/1120148
local locals = function(level)
    local variables = {}
    local idx = 1
    while true do
        local ln, lv = debug.getlocal(level, idx)
        if ln ~= nil then
            if lv ~= nil then
                variables[ln] = lv
            else
                variables[ln] = 0
            end
        else
            break
        end
        idx = 1 + idx
    end
    return variables
end

local upvalues = function(level)
    local variables = {}
    local idx = 1
    local func = debug.getinfo(level, "f").func
    while true do
        local ln, lv = debug.getupvalue(func, idx)
        if ln ~= nil then
            if lv ~= nil then
                variables[ln] = lv
            else
                variables[ln] = 0
            end
        else
            break
        end
        idx = 1 + idx
    end
    return variables
end

local getEvalENV = function(locals, upvalues, globals)
    return setmetatable({},{
        __index = function(self, key)
            if locals[key] ~= nil then
                return locals[key]
            elseif upvalues[key] ~= nil then
                return upvalues[key]
            else
                return globals[key]
            end
        end,
        __newindex = function(self, key, value)
            if locals[key] ~= nil then
                locals[key] = value
            elseif upvalues[key] ~= nil then
                upvalues[key] = value
            else
                globals[key] = value
            end
        end
    })
end

local evalLuaString = function(str, _G)
    local level = 4

    -- collect locals and upvalues
    local _l = locals(level)
    local _u = upvalues(level)
    local _evalENV = getEvalENV(_l, _u, _G)

    -- eval lua code
    local evaluated = assert(load(str, nil, "t", _evalENV))
    local last, beforeLast
    local catch = function()
        beforeLast = last
        local _, v = debug.getlocal(2, 1)
        last = v
    end

    -- catch the last return
    debug.sethook(catch, "r")
    evaluated()
    debug.sethook()

    -- set upvalues
    local _idx = 1
    local _func = debug.getinfo(level - 1, "f").func
    while true do
        local ln = debug.getupvalue(_func, _idx)
        if ln ~= nil then
            if ln ~= "(*temporary)" and _u[ln] ~= nil then
                debug.setupvalue(_func, _idx, _u[ln])
            end

        else
            break
        end
        _idx = 1 + _idx
    end

    -- set locals
    _idx = 1
    while true do
        local ln = debug.getlocal(level - 1, _idx)
        if ln ~= nil then
            if ln ~= "(*temporary)" and _l[ln] ~= nil then
                debug.setlocal(level - 1, _idx, _l[ln])
            end
        else
            break
        end
        _idx = 1 + _idx
    end

    return beforeLast
end

function eval.eval(this, str)
    if type(str) ~= "string" then
        return str
    end

    if luajit then
        castl = castl or require("castl.jscompile.castl_jit")
        esprima = esprima or require("castl.jscompile.esprima_jit")
    else
        castl = castl or require("castl.jscompile.castl")
        esprima = esprima or require("castl.jscompile.esprima")
    end

    -- parse and compile JS code
    local ast = esprima:parse(str)
    -- castl used in eval mode
    local castlOptions = {jit = luajit, evalMode = true}
    local castlResult = castl:compileAST(ast, castlOptions)
    local ret

    if castlResult.success then
        local luaCode = castlResult.compiled
        runtime = runtime or require("castl.runtime")
        ret = evalLuaString(luaCode, runtime)
    else
        throw(errorHelper.newEvalError("Eval(): Failed to compile AST to Lua code"))
    end

    return ret
end

return eval

end
end

do
local _ENV = _ENV
package.preload[ "castl.global_functions" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local globalFunctions = {}

local internal = require("castl.internal")

-- Dependencies
local null, ToNumber, ToString = internal.null, internal.ToNumber, internal.ToString
local type, tonumber = type, tonumber
local huge, floor, abs, ceil = math.huge, math.floor, math.abs, math.ceil
local gsub, sub, match, format, sbyte, find, char = string.gsub, string.sub, string.match, string.format, string.byte, string.find, string.char

_ENV = nil

function globalFunctions.isNaN(this, n)
    local tonum = ToNumber(n)
    return tonum ~= tonum
end

function globalFunctions.isFinite(this, arg)
    arg = ToNumber(arg)
    return arg ~= huge and arg ~= -huge and not (arg ~= arg)
end

function globalFunctions.parseFloat(this, str)
    local v = tonumber(ToString(str))
    if v == nil then
        return 0/0
    else
        return v
    end
end

-- http://www.ecma-international.org/ecma-262/5.1/#sec-15.1.2.2
local radixDigit = {
    [2] = "[^01]",
    [3] = "[^012]",
    [4] = "[^0123]",
    [5] = "[^01234]",
    [6] = "[^012345]",
    [7] = "[^0123456]",
    [8] = "[^01234567]",
    [9] = "[^012345678]",
    [10] = "[^0123456789]",
    [11] = "[^0123456789aA]",
    [12] = "[^0123456789aAbB]",
    [13] = "[^0123456789aAbBcC]",
    [14] = "[^0123456789aAbBcCdD]",
    [15] = "[^0123456789aAbBcCdDeE]",
    [16] = "[^0123456789aAbBcCdDeEfF]",
    [17] = "[^0123456789aAbBcCdDeEfFgG]",
    [18] = "[^0123456789aAbBcCdDeEfFgGhH]",
    [19] = "[^0123456789aAbBcCdDeEfFgGhHiI]",
    [20] = "[^0123456789aAbBcCdDeEfFgGhHiIjJ]",
    [21] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkK]",
    [22] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlL]",
    [23] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmM]",
    [24] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnN]",
    [25] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoO]",
    [26] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpP]",
    [27] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQ]",
    [28] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrR]",
    [29] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsS]",
    [30] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStT]",
    [31] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuU]",
    [32] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvV]",
    [33] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwW]",
    [34] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxX]",
    [35] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyY]",
    [36] = "[^0123456789aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ]"
}

function globalFunctions.parseInt(this, str, radix)
    radix = radix or 10
    local tstr = type(str)

    -- shortcut: parseInt used as a cast to int
    if tstr == "number" and radix == 10 then
        return str < 0 and ceil(str) or floor(str)
    end

    -- radix check
    radix = tonumber(radix)
    if radix ~= 0 then
        -- radix bounds
        if radix < 2 or radix > 36 then
            return 0/0
        end
    else
        -- if radix == 0 then radix is treated as 10
        radix = 10
    end

    local sign = 1
    if tstr ~= "number" then
        str = ToString(str)

        -- trim
        str = match(str,'^()%s*$') and '' or match(str,'^%s*(.*%S)')

        if str ~= "" then
            if sub(str, 1, 1) == "-" then
                sign = -1
            end
            str = gsub(str, "^[%+%-]", "")
        end

        if radix == 16 then
            -- strip hexa prefix 0x if it exists
            str = gsub(str, "^0[xX]", "")
        end

        -- If str contains any character that is not a radix digit,
        -- then str becomes the substring of str consisting of all characters before the first such character
        local firstUnallowedCharOffset = (find(str, radixDigit[radix]))
        if firstUnallowedCharOffset then
            str = sub(str, 1, firstUnallowedCharOffset - 1)
        end
    else
        -- str is a number
        sign = (str > 0) and 1 or -1
        str = floor(abs(str))
    end

    local v = tonumber(str, radix)
    if v then
        return sign * v
    end

    return 0/0
end

-- URI
-- http://www.ecma-international.org/ecma-262/5.1/#sec-15.1.3

local uriReserved = ";/%?:@&=%+%$,"
local uriMark = "%-_%.!~*'%(%)"
local uriUnescaped = "%a%d" .. uriMark
local score = "#"

-- Encoding

local patternEncodeURI = "[^" .. uriReserved .. uriUnescaped .. score  .. "]"
local patternEncodeURIComponent = "[^" .. uriUnescaped .. "]"

local encode = function(str, unescapedSet)
    return (gsub(str, unescapedSet,
        function(c)
            return format("%%%02X", sbyte(c))
        end
    ))
end

globalFunctions.encodeURI = function (this, uri)
    uri = ToString(uri)
    return encode(uri, patternEncodeURI)
end

globalFunctions.encodeURIComponent = function (this, uri)
    uri = ToString(uri)
    return encode(uri, patternEncodeURIComponent)
end

-- Decoding

local decode = function(str, reservedSet)
    return (gsub(str, "%%(%x%x)",
        function(h)
            local char = char(tonumber(h, 16))
            if find(reservedSet, char, 1, true) then
                return "%" .. h
            end
            return char
        end
    ))
end

local reservedURISet = ";/?:@&=+$,#"

globalFunctions.decodeURI = function (this, uri)
    uri = ToString(uri)
    return decode(uri, reservedURISet)
end

globalFunctions.decodeURIComponent = function (this, uri)
    uri = ToString(uri)
    return decode(uri, "")
end

return globalFunctions

end
end

do
local _ENV = _ENV
package.preload[ "castl.internal" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local internal = {}

local errorHelper = require("castl.modules.error_helper")

local Boolean, Number, String, new, objectToString, obj, throw

local getmetatable, setmetatable, type, tostring, tonumber, require, rawget, rawset = getmetatable, setmetatable, type, tostring, tonumber, require, rawget, rawset
local gsub = string.gsub
local floor, ceil = math.floor, math.ceil

_ENV = nil

-- forward declaration of null
internal.null = {}
local null = internal.null

-- Functions proxy
local functionsProxyObjects = {}
setmetatable(functionsProxyObjects, {__mode = 'k'})

function internal.getFunctionProxy(fun)
    local proxy = rawget(functionsProxyObjects, fun)
    if proxy == nil then
        obj = obj or require("castl.core_objects").obj
        proxy = {prototype = obj({constructor = fun})}
        rawset(functionsProxyObjects, fun, proxy)
    end
    return proxy
end

local getFunctionProxy = internal.getFunctionProxy

function internal.isPrimitiveValue(v)
    local tv = type(v)
    return tv == "string" or tv == "number" or tv == "boolean" or v == null or v == nil
end

local isPrimitiveValue = internal.isPrimitiveValue

-- http://www.ecma-international.org/ecma-262/5.1/#sec-8.12.8
function internal.defaultValueString(o)
    if type(o.toString) == "function" then
        local value = o:toString()
        if isPrimitiveValue(value) then
            return value
        end
    end
    if type(o.valueOf) == "function" then
        local value = o:valueOf()
        if isPrimitiveValue(value) then
            return value
        end
    end

    throw = throw or require("castl.jssupport").throw
    throw(errorHelper.newTypeError("Cannot convert object to primitive value"))
end

function internal.defaultValueNumber(o)
    if type(o.valueOf) == "function" then
        local value = o:valueOf()
        if isPrimitiveValue(value) then
            return value
        end
    end
    if type(o.toString) == "function" then
        local value = o:toString()
        if isPrimitiveValue(value) then
            return value
        end
    end

    throw = throw or require("castl.jssupport").throw
    throw(errorHelper.newTypeError("Cannot convert object to primitive value"))
end

local defaultValueNumber = internal.defaultValueNumber
local defaultValueString = internal.defaultValueString

function internal.ToPrimitiveString(o)
    if isPrimitiveValue(o) then
        return o
    end
    return defaultValueString(o)
end

function internal.ToPrimitiveNumber(o)
    if isPrimitiveValue(o) then
        return o
    end
    return defaultValueNumber(o)
end

local function ToNumber(value, typeV)
    local tvalue = typeV or type(value)
    if tvalue == "number" then
        return value
    elseif tvalue == "string" then
        if gsub(value, " ", "") == "" then return 0 end
        return tonumber(value) or 0/0
    elseif tvalue == "boolean" then
        return value and 1 or 0
    elseif value == null then
        return 0
    elseif value == nil then
        return 0/0
    end

    return ToNumber(defaultValueNumber(value))
end

internal.ToNumber = ToNumber

function internal.ToString(value)
    local tv = type(value)
    if tv == "string" then
        return value
    elseif tv == "number" or tv == "boolean" then
        return tostring(value)
    elseif value == null then
        return "null"
    elseif  value == nil then
        return "undefined"
    end

    return tostring(defaultValueString(value))
end

setmetatable(internal.null,{
    __index = function(self, key)
        throw = throw or require("castl.jssupport").throw
        throw(errorHelper.newTypeError("Cannot read property '" .. key .. "' of null"))
    end,
    __newindex = function(self, key)
        throw = throw or require("castl.jssupport").throw
        throw(errorHelper.newTypeError("Cannot set property '" .. key .. "' of null"))
    end,
    __tostring = function ()
        return 'null'
    end,
    __sub = function(a, b)
        return ToNumber(a) - ToNumber(b)
    end,
    __div = function(a, b)
        return ToNumber(a) / ToNumber(b)
    end,
    __mul = function(a, b)
        return ToNumber(a) * ToNumber(b)
    end,
    __tojson = function () return "null" end
})

-- get/put

-- manually goes along the prototype chain
function internal.get(self, prototype, key)
    local getterName, getter = "_g" .. tostring(key)

    -- self == nil => case of built-in types: boolean, string and number
    -- they are immutable, so we don't have to look for a getter
    if self ~= nil then
        if type(self) == "function" then self = getFunctionProxy(self) end
        getter = rawget(self, getterName)
        if getter then
            return getter(self)
        end
    end

    local current, att = prototype
    while current do
        if type(current) == "function" then current = getFunctionProxy(current) end
        -- try to get attribute
        att = rawget(current, key)
        if att ~= nil then
            return att
        end
        --try to get getter
        getter = rawget(current, getterName)
        if getter ~= nil then
            -- call getter with the good this (self, and not current)
            return getter(self)
        end
        current = (getmetatable(current) or {})._prototype
    end

    return nil
end

function internal.put(self, key, value)
    if key == nil then return end
    -- try to find a setter in
    local setterName, current, setter = "_s" .. tostring(key), self
    while current do
        if type(current) == "function" then current = getFunctionProxy(current) end
        setter = rawget(current, setterName)
        if setter ~= nil then
            -- call setter with the good this (self, and not current)
            setter(self, value)
            return;
        end
        current = (getmetatable(current) or {})._prototype
    end

    rawset(self, key, value)
end

local get, put = internal.get, internal.put

-- prototype
function internal.prototype(o)
    local mt = getmetatable(o)
    if mt then
        return mt._prototype
    end
    return nil
end

-- test if constructor called within a new
function internal.withinNew(this, proto)
    local mt = getmetatable(this)
    return mt and mt._prototype == proto
end

-- http://www.ecma-international.org/ecma-262/5.1/#sec-9.9
function internal.ToObject(v)
    if v == nil or v == null then
        throw = throw or require("castl.jssupport").throw
        throw(errorHelper.newTypeError("Cannot convert undefined or null to object"))
    end

    local tv = type(v)
    new = new or require("castl.core_objects").new
    if tv == "boolean" then
        Boolean = Boolean or require("castl.constructor.boolean")
        return new(Boolean, v)
    elseif tv == "number" then
        Number = Number or require("castl.constructor.number")
        return new(Number, v)
    elseif tv == "string" then
        String = String or require("castl.constructor.string")
        return new(String, v)
    end

    return v
end

-- create a metatable for a new object (used in new statement and Object.create)
function internal.setNewMetatable(o, prototype)
    local mt = {
        __index = function (self, key)
            return get(self, prototype, key)
        end,
        __newindex = put,
        __sub = function(a, b)
            return ToNumber(a) - ToNumber(b)
        end,
        __mul = function(a, b)
            return ToNumber(a) * ToNumber(b)
        end,
        __div = function(a, b)
            return ToNumber(a) / ToNumber(b)
        end,
        _prototype = prototype
    }

    local protoMt = getmetatable(prototype)
    if protoMt and protoMt.__tostring then
        mt.__tostring = protoMt.__tostring
    else
        objectToString = objectToString or require("castl.core_objects").objectToString
        mt.__tostring = objectToString
    end

    setmetatable(o, mt)
end

function internal.ToInteger(v)
    local number = ToNumber(v)
    return number < 0 and ceil(number) or floor(number)
end

function internal.ToUint32(v)
    local number = ToNumber(v)
    return number < 0 and ceil(number) % 0x100000000 or floor(number) % 0x100000000
end

function internal.ToUint16(v)
    local number = ToNumber(v)
    return number < 0 and ceil(number) % 0x10000 or floor(number) % 0x10000
end

return internal

end
end

do
local _ENV = _ENV
package.preload[ "castl.json" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL others submodule]] --

local coreObjects = require("castl.core_objects")
local internal = require("castl.internal")
local json = require("castl.modules.dkjson")
local errorHelper = require("castl.modules.error_helper")
local throw = require("castl.jssupport").throw

local null, ToString = internal.null, internal.ToString

local JSON = {}

_ENV = nil

JSON.parse = function(this, text)
    text = ToString(text)
    local obj, pos, err = json.decode(text, 1, null, coreObjects.objectMt, coreObjects.arrayMt)
    if err then
        throw(errorHelper.newSyntaxError(err))
    end

    return obj
end

JSON.stringify = function(this, value)
    if value == nil then return nil end
    return json.encode(value)
end

return JSON

end
end

do
local _ENV = _ENV
package.preload[ "castl.jssupport" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL JS Support submodule]] --

local jssupport = {}

local errorHelper = require("castl.modules.error_helper")
local internal = require("castl.internal")
local luajit = jit ~= nil

-- Dependencies
local null, ToPrimitiveNumber, ToNumber, ToObject, ToString = internal.null, internal.ToPrimitiveNumber, internal.ToNumber, internal.ToObject, internal.ToString
local type, tonumber, tostring, pairs, setmetatable, getmetatable, error, select = type, tonumber, tostring, pairs, setmetatable, getmetatable, error, select
local tinsert = table.insert
local huge, abs = math.huge, math.abs
local pcall = pcall

-- Prevent modification of global environment
_ENV = nil

jssupport.throw = function(e)
    -- Error.captureStackTrace was called on e
    -- will display captured stack if uncaught
    if (getmetatable(e) or {}).__stack then
        getmetatable(e).__tostring = function() return e:_gstack() end
    end

    error(e, 0)
end

local throw = jssupport.throw

jssupport.void = function() end
jssupport.e = function(...) return ... end
jssupport.Infinity = huge

-- Define custom break and continue (cf try statement inside a loop)
jssupport._break = {}
jssupport._continue = {}

-- http://www.ecma-international.org/ecma-262/5.1/#sec-11.9.3
local equal
equal = function(x, y)
    local tx, ty = type(x), type(y)

    -- case 1
    if tx == ty then
        return x == y
    end
    -- case 2
    if x == null and ty == "nil" then return true end
    -- case 3
    if tx == "nil" and y == null then return true end
    -- case 4
    if tx == "number" and ty == "string" then
        return equal(x, tonumber(y))
    end
    -- case 5
    if tx == "string" and ty == "number" then
        return equal(tonumber(x), y)
    end
    -- case 6
    if tx == "boolean" then return equal(x and 1 or 0, y) end
    -- case 7
    if ty == "boolean" then return equal(x, y and 1 or 0) end
    -- case 8
    if (tx == "string" or tx == "number") and (ty == "table" and y ~= null) then
        return equal(x, ToPrimitiveNumber(y))
    end
    -- case 9
    if (tx == "table" and x ~= null) and (ty == "string" or ty == "number") then
        return equal(ToPrimitiveNumber(x), y)
    end
    -- case 10
    return false
end

jssupport.equal = equal

-- Avoid Lua coercion
function jssupport.add(x, y)
    x, y = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
    local tx, ty = type(x), type(y)
    if tx == "string" or ty == "string" then
        return tostring(x) .. tostring(y)
    else
        return ToNumber(x, tx) + ToNumber(y, ty)
    end
end

-- first param is known to be a string, second is unknown
function jssupport.addString1(str, other)
    return str .. tostring(ToPrimitiveNumber(other))
end

-- first param is unknown, second is known to be a string
function jssupport.addString2(other, str)
    return tostring(ToPrimitiveNumber(other)) .. str
end

-- first param is known to be a number, second is unknown
function jssupport.addNumber1(num, other)
    other = ToPrimitiveNumber(other)
    local tother = type(other)
    if tother == "string" then
        return num .. tostring(other)
    else
        return num + ToNumber(other, tother)
    end
end

-- first param is unknown, second is known to be a number
function jssupport.addNumber2(other, num)
    other = ToPrimitiveNumber(other)
    local tother = type(other)
    if tother == "string" then
        return tostring(other) .. num
    else
        return ToNumber(other, tother) + num
    end
end

if luajit then
    function jssupport.lt(x, y)
        local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
        local tpx, tpy = type(px), type(py)
        if tpx == "string" and tpy == "string" then
            return px < py
        end

        return ToNumber(px, tpx) < ToNumber(py, tpy)
    end

    function jssupport.le(x, y)
        local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
        local tpx, tpy = type(px), type(py)
        if tpx == "string" and tpy == "string" then
            return px <= py
        end

        return ToNumber(px, tpx) <= ToNumber(py, tpy)
    end

    function jssupport.gt(x, y)
        local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
        local tpx, tpy = type(px), type(py)
        if tpx == "string" and tpy == "string" then
            return px > py
        end

        return ToNumber(px, tpx) > ToNumber(py, tpy)
    end

    function jssupport.ge(x, y)
        local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
        local tpx, tpy = type(px), type(py)
        if tpx == "string" and tpy == "string" then
            return px >= py
        end

        return ToNumber(px, tpx) >= ToNumber(py, tpy)

    end
else
    function jssupport.lt(x, y)
        local status, value = pcall(function() return x < y end)
        if status then return value
        else
            local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
            local tpx, tpy = type(px), type(py)
            if tpx == "string" and tpy == "string" then
                return px < py
            end

            return ToNumber(px, tpx) < ToNumber(py, tpy)
        end
    end

    function jssupport.le(x, y)
        local status, value = pcall(function() return x <= y end)
        if status then return value
        else
            local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
            local tpx, tpy = type(px), type(py)
            if tpx == "string" and tpy == "string" then
                return px <= py
            end

            return ToNumber(px, tpx) <= ToNumber(py, tpy)
        end
    end

    function jssupport.gt(x, y)
        local status, value = pcall(function() return x > y end)
        if status then return value
        else
            local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
            local tpx, tpy = type(px), type(py)
            if tpx == "string" and tpy == "string" then
                return px > py
            end

            return ToNumber(px, tpx) > ToNumber(py, tpy)
        end
    end

    function jssupport.ge(x, y)
        local status, value = pcall(function() return x >= y end)
        if status then return value
        else
            local px, py = ToPrimitiveNumber(x), ToPrimitiveNumber(y)
            local tpx, tpy = type(px), type(py)
            if tpx == "string" and tpy == "string" then
                return px >= py
            end

            return ToNumber(px, tpx) >= ToNumber(py, tpy)
        end
    end
end

function jssupport.inc(x)
    return ToNumber(x) + 1
end

function jssupport.dec(x)
    return ToNumber(x) - 1
end

-- 0, NaN and "" are true in Lua and false in JS
function jssupport.boolean(var)
    if var == 0 or var == "" or var == null or var ~= var then
        return false
    else
        return var
    end
end

function jssupport.modulo(a, b)
    a, b  = ToNumber(a), ToNumber(b)
    local sign = a > 0 and 1 or -1
    return sign * (abs(a) % abs(b))
end

function jssupport.typeof(var)
    if var == nil then
        return "undefined"
    end
    local tvar = type(var)
    if tvar == 'table' then
        return "object"
    end
    return tvar
end

function jssupport.inOp (object, key)
    return object[key] ~= nil
end

function jssupport.with(obj, env)
    local copy = {}

    if obj == nil then
        throw(errorHelper.newTypeError("undefined has no properties"))
    elseif obj == null then
        throw(errorHelper.newTypeError("null has no properties"))
    end

    obj = ToObject(obj)

    -- return new environment
    return setmetatable({}, {
        __index = function(self, key)
            if obj[key] ~= nil then
                if type(obj[key]) == "function" then
                    return function(...)
                        -- 'this' argument of the function is the object
                        -- thus the original 'this' is discarded
                        return obj[key](obj, select(2, ...))
                    end
                end
                return obj[key]
            end

            return env[key]
        end,
        __newindex = function(self, key, value)
            if obj[key] ~= nil then
                obj[key] = value
            else
                env[key] = value
            end
        end})
end

return jssupport

end
end

do
local _ENV = _ENV
package.preload[ "castl.math" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Math object submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/abs

local Math = {}

local internal = require("castl.internal")
local coreObjects = require("castl.core_objects")

local math, tonumber, time = math, tonumber, os.time
local null = internal.null

_ENV = nil

-- init random
math.randomseed(time())

-- constants
Math.E = math.exp(1)
Math.LN2 = math.log(2)
Math.LN10 = math.log(10)
Math.LOG2E = 1 / math.log(2)
Math.LOG10E = 1 / math.log(10)
Math.PI = math.pi
Math.SQRT1_2 = 1 / math.sqrt(2)
Math.SQRT2 = math.sqrt(2)

-- functions

local san = function(x)
    return ((x == null) and 0 or tonumber(x)) or 0/0
end

Math.abs = function(this, x) return math.abs(san(x)) end
Math.acos = function(this,x) return math.acos(san(x)) end
Math.asin = function(this,x) return math.asin(san(x)) end
Math.atan = function(this,x) return math.atan(san(x)) end
Math.atan2 = function(this,x,y) return math.atan2(san(x),san(y)) end
Math.ceil = function(this,x) return math.ceil(san(x)) end
Math.cos = function(this,x) return math.cos(san(x)) end
Math.exp = function(this,x) return math.exp(san(x)) end
Math.floor = function(this,x) return math.floor(san(x)) end
Math.log = function(this,x) return math.log(san(x)) end
Math.max = function(this,x,y) return math.max(san(x), san(y)) end
Math.min = function(this,x,y) return math.min(san(x), san(y)) end
Math.pow = function(this,x,y) return (san(x) ^ san(y)) end
Math.random = function(this) return math.random() end
Math.round = function(this, x) return math.floor(san(x) + 0.5) end
Math.sin = function(this,x) return math.sin(san(x)) end
Math.sqrt = function(this,x) return math.sqrt(san(x)) end
Math.tan = function(this,x) return math.tan(san(x)) end

return coreObjects.obj(Math)

end
end

do
local _ENV = _ENV
package.preload[ "castl.modules.bit" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local luajit = jit ~= nil
local band, bor, bnot, bxor, lshift, rshift

-- if executed by LuaJIT use its bit library as base
if luajit then
    band, bor, bnot, bxor, lshift, rshift = bit.band, bit.bor, bit.bnot, bit.bxor, bit.lshift, bit.rshift
else
    -- else use bit32 lib of Lua 5.2
    band, bor, bnot, bxor, lshift, rshift = bit32.band, bit32.bor, bit32.bnot, bit32.bxor, bit32.lshift, bit32.rshift
end

local bit = {}

local ToUint32 = require("castl.internal").ToUint32
local ToInteger = require("castl.internal").ToInteger

_ENV = nil

bit.lshift = function(x, disp)
    x, disp = ToInteger(x), ToUint32(disp)
    local shiftCount = band(disp, 0x1F)
    local ret = lshift(x, shiftCount);

    -- Ones' complement
    if band(ret, 0x80000000) > 0 then
        ret = -(bnot(ret) + 1)
    end

    return ret
end

bit.rshift = function(x, disp)
    x, disp = ToInteger(x), ToUint32(disp)
    if luajit and disp == 0 then
        return x % 0x100000000
    end

    local shiftCount = band(disp, 0x1F)
    return rshift(x, shiftCount)
end

bit.arshift = function(x, disp)
    x, disp = ToInteger(x), ToUint32(disp)
    local shiftCount = band(disp, 0x1F)
    local ret = rshift(x, shiftCount)

    if x < 0 then
        for i = 31, 31 - shiftCount, -1 do
            -- set bit to 1
            ret = bor(ret, lshift(1,i))
        end
        -- Ones' complement
        ret = -(bnot(ret) + 1)
    end

    return ret
end

bit.band = function(x, y)
    return band(ToInteger(x), ToInteger(y))
end

bit.bor = function(x, y)
    return bor(ToInteger(x), ToInteger(y))
end

bit.bxor = function(x, y)
    return bxor(ToInteger(x), ToInteger(y))
end

bit.bnot = function(x)
    return bnot(ToInteger(x))
end

return bit

end
end

do
local _ENV = _ENV
package.preload[ "castl.modules.dateparser" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local difftime, time, date = os.difftime, os.time, os.date
local tinsert = table.insert
local pcall, ipairs, tostring, tonumber, type, setmetatable = pcall, ipairs, tostring, tonumber, type, setmetatable
local match = string.match

local DateParser = {}

_ENV = nil

--we shall use the host OS's time conversion facilities. Dealing with all those leap seconds by hand can be such a bore.
local unix_timestamp
do
    local now = time()
    local local_UTC_offset_sec = difftime(time(date("!*t", now)), time(date("*t", now)))
    unix_timestamp = function(t, offset_sec)
        local success, improper_time = pcall(time, t)
        if not success or not improper_time then return nil, "invalid date. os.time says: " .. (improper_time or "nothing") end
        return improper_time - local_UTC_offset_sec - offset_sec
    end
end

local formats = {} -- format names
local format_func = setmetatable({}, {__mode='v'})  --format functions

---register a date format parsing function
local register_format = function(format_name, format_function)
    if type(format_name)~="string" or type(format_function)~='function' then return nil, "improper arguments, can't register format handler" end

    local found
    for i, f in ipairs(format_func) do --for ordering
        if f==format_function then
            found=true
            break
    end
    end
    if not found then
        tinsert(format_func, format_function)
    end
    formats[format_name] = format_function
    return true
end

---try to parse date string
--@param str date string
--@param date_format optional date format name, if known
--@return unix timestamp if str can be parsed; nil, error otherwise.
DateParser.parse = function(str, date_format)
    local success, res
    if date_format then
        if not formats[date_format] then return 'unknown date format: ' .. tostring(date_format) end
        success, res = pcall(formats[date_format], str)
    else
        for i, func in ipairs(format_func) do
            success, res = pcall(func, str)
            if success and res then return res end
        end
    end

    return success and res
end


do
    local tz_table = { --taken from http://www.timeanddate.com/library/abbreviations/timezones/
        A = 1,  B = 2, C = 3, D = 4,  E=5, F = 6,   G = 7,  H = 8,  I = 9,
        K = 10, L = 11, M = 12, N = -1, O = -2, P = -3, Q = -4, R = -5,
        S = -6, T = -7, U = -8, V = -9, W = -10, X = -11, Y = -12,
        Z = 0,

        EST = -5, EDT = -4, CST = -6, CDT = -5,
        MST = -7, MDT = -6, PST = -8, PDT = -7,

        GMT = 0, UT = 0, UTC = 0
    }

    local month_val = {Jan=1, Feb=2, Mar=3, Apr=4, May=5, Jun=6, Jul=7, Aug=8, Sep=9, Oct=10, Nov=11, Dec=12}

    register_format('RFC2822', function(rest)
        local year, month, day, day_of_year, week_of_year, weekday
        local hour, minute, second, second_fraction, offset_hours

        local alt_rest

        weekday, alt_rest = match(rest, "^(%w%w%w),%s+(.*)$")
        if weekday then rest=alt_rest end
        day, rest=match(rest,"^(%d%d?)%s+(.*)$")
        month, rest=match(rest,"^(%w%w%w)%s+(.*)$")
        month = month_val[month]
        year, rest = match(rest,"^(%d%d%d?%d?)%s+(.*)$")
        hour, rest = match(rest,"^(%d%d?):(.*)$")
        minute, rest = match(rest,"^(%d%d?)(.*)$")
        second, alt_rest = match(rest,"^:(%d%d)(.*)$")
        if second then rest = alt_rest end
        local tz, offset_sign, offset_h, offset_m
        tz, alt_rest = match(rest,"^%s+(%u+)(.*)$")
        if tz then
            rest = alt_rest
            offset_hours = tz_table[tz]
        else
            offset_sign, offset_h, offset_m, rest = match(rest,"^%s+([+-])(%d%d)(%d%d)%s*(.*)$")
            offset_hours = tonumber(offset_sign .. offset_h) + (tonumber(offset_m) or 0)/60
        end

        if #rest>0 or not (year and day and month and hour and minute) then
            return nil
        end

        year = tonumber(year)
        local d = {
            year = year and ((year > 100) and year or (year < 50 and (year + 2000) or (year + 1900))),
            month = month,
            day = tonumber(day),

            hour= tonumber(hour) or 0,
            min = tonumber(minute) or 0,
            sec = tonumber(second) or 0,
            isdst  = false
        }
        return unix_timestamp(d, offset_hours * 3600)
    end)
end

return DateParser

end
end

do
local _ENV = _ENV
package.preload[ "castl.modules.dkjson" ] = function( ... ) local arg = _G.arg;
-- modified version of dkjson 2.5
-- to handle custom castl object scheme

-- Module options:
local always_try_using_lpeg = true
local register_global_module_table = false
local global_module_name = 'json'

local arrayProto = require("castl.protos").arrayProto

--[==[

David Kolf's JSON module for Lua 5.1/5.2

Version 2.5


For the documentation see the corresponding readme.txt or visit
<http://dkolf.de/src/dkjson-lua.fsl/>.

You can contact the author by sending an e-mail to 'david' at the
domain 'dkolf.de'.


Copyright (C) 2010-2013 David Heiko Kolf

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

--]==]

-- global dependencies:
local pairs, type, tostring, tonumber, getmetatable, setmetatable, rawset  =
    pairs, type, tostring, tonumber, getmetatable, setmetatable, rawset 
local error, require, pcall, select = error, require, pcall, select
local floor, huge = math.floor, math.huge
local strrep, gsub, strsub, strbyte, strchar, strfind, strlen, strformat =
    string.rep, string.gsub, string.sub, string.byte, string.char,
    string.find, string.len, string.format
local strmatch = string.match
local concat = table.concat
local rawget, tremove = rawget, table.remove

local json = { version = "dkjson 2.5" }

if register_global_module_table then
    _G[global_module_name] = json
end

local _ENV = nil -- blocking globals in Lua 5.2

pcall (function()
    -- Enable access to blocked metatables.
    -- Don't worry, this module doesn't change anything in them.
    local debmeta = require "debug".getmetatable
    if debmeta then getmetatable = debmeta end
end)

json.null = setmetatable ({}, {
    __tojson = function () return "null" end
})

-- CASTL modification
local function isarray (tbl)
    local b = (getmetatable(tbl) or {})._prototype == arrayProto
    return b and true, tbl.length or false
end

local escapecodes = {
    ["\""] = "\\\"", ["\\"] = "\\\\", ["\b"] = "\\b", ["\f"] = "\\f",
    ["\n"] = "\\n",  ["\r"] = "\\r",  ["\t"] = "\\t"
}

local function escapeutf8 (uchar)
    local value = escapecodes[uchar]
    if value then
        return value
    end
    local a, b, c, d = strbyte (uchar, 1, 4)
    a, b, c, d = a or 0, b or 0, c or 0, d or 0
    if a <= 0x7f then
        value = a
    elseif 0xc0 <= a and a <= 0xdf and b >= 0x80 then
        value = (a - 0xc0) * 0x40 + b - 0x80
    elseif 0xe0 <= a and a <= 0xef and b >= 0x80 and c >= 0x80 then
        value = ((a - 0xe0) * 0x40 + b - 0x80) * 0x40 + c - 0x80
    elseif 0xf0 <= a and a <= 0xf7 and b >= 0x80 and c >= 0x80 and d >= 0x80 then
        value = (((a - 0xf0) * 0x40 + b - 0x80) * 0x40 + c - 0x80) * 0x40 + d - 0x80
    else
        return ""
    end
    if value <= 0xffff then
        return strformat ("\\u%.4x", value)
    elseif value <= 0x10ffff then
        -- encode as UTF-16 surrogate pair
        value = value - 0x10000
        local highsur, lowsur = 0xD800 + floor (value/0x400), 0xDC00 + (value % 0x400)
        return strformat ("\\u%.4x\\u%.4x", highsur, lowsur)
    else
        return ""
    end
end

local function fsub (str, pattern, repl)
    -- gsub always builds a new string in a buffer, even when no match
    -- exists. First using find should be more efficient when most strings
    -- don't contain the pattern.
    if strfind (str, pattern) then
        return gsub (str, pattern, repl)
    else
        return str
    end
end

local function quotestring (value)
    -- based on the regexp "escapable" in https://github.com/douglascrockford/JSON-js
    value = fsub (value, "[%z\1-\31\"\\\127]", escapeutf8)
    if strfind (value, "[\194\216\220\225\226\239]") then
        value = fsub (value, "\194[\128-\159\173]", escapeutf8)
        value = fsub (value, "\216[\128-\132]", escapeutf8)
        value = fsub (value, "\220\143", escapeutf8)
        value = fsub (value, "\225\158[\180\181]", escapeutf8)
        value = fsub (value, "\226\128[\140-\143\168-\175]", escapeutf8)
        value = fsub (value, "\226\129[\160-\175]", escapeutf8)
        value = fsub (value, "\239\187\191", escapeutf8)
        value = fsub (value, "\239\191[\176-\191]", escapeutf8)
    end
    return "\"" .. value .. "\""
end
json.quotestring = quotestring

local function replace(str, o, n)
    local i, j = strfind (str, o, 1, true)
    if i then
        return strsub(str, 1, i-1) .. n .. strsub(str, j+1, -1)
    else
        return str
    end
end

-- locale independent num2str and str2num functions
local decpoint, numfilter

local function updatedecpoint ()
    decpoint = strmatch(tostring(0.5), "([^05+])")
    -- build a filter that can be used to remove group separators
    numfilter = "[^0-9%-%+eE" .. gsub(decpoint, "[%^%$%(%)%%%.%[%]%*%+%-%?]", "%%%0") .. "]+"
end

updatedecpoint()

local function num2str (num)
    return replace(fsub(tostring(num), numfilter, ""), decpoint, ".")
end

local function str2num (str)
    local num = tonumber(replace(str, ".", decpoint))
    if not num then
        updatedecpoint()
        num = tonumber(replace(str, ".", decpoint))
    end
    return num
end

local function addnewline2 (level, buffer, buflen)
    buffer[buflen+1] = "\n"
    buffer[buflen+2] = strrep ("  ", level)
    buflen = buflen + 2
    return buflen
end

function json.addnewline (state)
    if state.indent then
        state.bufferlen = addnewline2 (state.level or 0,
            state.buffer, state.bufferlen or #(state.buffer))
    end
end

local encode2 -- forward declaration

local function addpair (key, value, prev, indent, level, buffer, buflen, tables, globalorder, state)
    local kt = type (key)
    if kt ~= 'string' and kt ~= 'number' then
        return nil, "type '" .. kt .. "' is not supported as a key by JSON."
    end
    
    if type(value) == "function" then
        return buflen
    end
    
    if prev then
        buflen = buflen + 1
        buffer[buflen] = ","
    end
    if indent then
        buflen = addnewline2 (level, buffer, buflen)
    end
    buffer[buflen+1] = quotestring (key)
    buffer[buflen+2] = ":"
    return encode2 (value, indent, level, buffer, buflen + 2, tables, globalorder, state)
end

local function appendcustom(res, buffer, state)
    local buflen = state.bufferlen
    if type (res) == 'string' then
        buflen = buflen + 1
        buffer[buflen] = res
    end
    return buflen
end

local function exception(reason, value, state, buffer, buflen, defaultmessage)
    defaultmessage = defaultmessage or reason
    local handler = state.exception
    if not handler then
        return nil, defaultmessage
    else
        state.bufferlen = buflen
        local ret, msg = handler (reason, value, state, defaultmessage)
        if not ret then return nil, msg or defaultmessage end
        return appendcustom(ret, buffer, state)
    end
end

function json.encodeexception(reason, value, state, defaultmessage)
    return quotestring("<" .. defaultmessage .. ">")
end

encode2 = function (value, indent, level, buffer, buflen, tables, globalorder, state)
    local valtype = type (value)
    local valmeta = getmetatable (value)
    valmeta = type (valmeta) == 'table' and valmeta -- only tables
    local valtojson = valmeta and valmeta.__tojson
    if valtojson then
        if tables[value] then
            return exception('reference cycle', value, state, buffer, buflen)
        end
        tables[value] = true
        state.bufferlen = buflen
        local ret, msg = valtojson (value, state)
        if not ret then return exception('custom encoder failed', value, state, buffer, buflen, msg) end
        tables[value] = nil
        buflen = appendcustom(ret, buffer, state)
    elseif value == nil then
        buflen = buflen + 1
        buffer[buflen] = "null"
    elseif valtype == 'number' then
        local s
        if value ~= value or value >= huge or -value >= huge then
            -- This is the behaviour of the original JSON implementation.
            s = "null"
        else
            s = num2str (value)
        end
        buflen = buflen + 1
        buffer[buflen] = s
    elseif valtype == 'boolean' then
        buflen = buflen + 1
        buffer[buflen] = value and "true" or "false"
    elseif valtype == 'string' then
        buflen = buflen + 1
        buffer[buflen] = quotestring (value)
    elseif valtype == 'table' then
        if tables[value] then
            return exception('reference cycle', value, state, buffer, buflen)
        end
        tables[value] = true
        level = level + 1
        local isa, n = isarray (value)
        if n == 0 and valmeta and valmeta.__jsontype == 'object' then
            isa = false
        end
        local msg
        if isa then -- JSON array
            buflen = buflen + 1
            buffer[buflen] = "["
            -- CASTL modification: change bounds
            for i = 0, n - 1 do
                buflen, msg = encode2 (value[i], indent, level, buffer, buflen, tables, globalorder, state)
                if not buflen then return nil, msg end
                if i < n - 1 then
                    buflen = buflen + 1
                    buffer[buflen] = ","
                end
            end
            buflen = buflen + 1
            buffer[buflen] = "]"
        else -- JSON object
            local prev = false
            buflen = buflen + 1
            buffer[buflen] = "{"
            local order = valmeta and valmeta.__jsonorder or globalorder
            if order then
                local used = {}
                n = #order
                for i = 1, n do
                    local k = order[i]
                    local v = value[k]
                    if v then
                        used[k] = true
                        buflen, msg = addpair (k, v, prev, indent, level, buffer, buflen, tables, globalorder, state)
                        prev = true -- add a seperator before the next element
                    end
                end
                for k,v in pairs (value) do
                    if not used[k] then
                        buflen, msg = addpair (k, v, prev, indent, level, buffer, buflen, tables, globalorder, state)
                        if not buflen then return nil, msg end
                        prev = true -- add a seperator before the next element
                    end
                end
            else -- unordered
                for k,v in pairs (value) do
                    buflen, msg = addpair (k, v, prev, indent, level, buffer, buflen, tables, globalorder, state)
                    if not buflen then return nil, msg end
                    prev = true -- add a seperator before the next element
            end
            end
            if indent then
                buflen = addnewline2 (level - 1, buffer, buflen)
            end
            buflen = buflen + 1
            buffer[buflen] = "}"
        end
        tables[value] = nil
    else
        return exception ('unsupported type', value, state, buffer, buflen,
            "type '" .. valtype .. "' is not supported by JSON.")
    end
    return buflen
end

function json.encode (value, state)
    state = state or {}
    local oldbuffer = state.buffer
    local buffer = oldbuffer or {}
    state.buffer = buffer
    updatedecpoint()
    local ret, msg = encode2 (value, state.indent, state.level or 0,
        buffer, state.bufferlen or 0, state.tables or {}, state.keyorder, state)
    if not ret then
        error (msg, 2)
    elseif oldbuffer == buffer then
        state.bufferlen = ret
        return true
    else
        state.bufferlen = nil
        state.buffer = nil
        return concat (buffer)
    end
end

local function loc (str, where)
    local line, pos, linepos = 1, 1, 0
    while true do
        pos = strfind (str, "\n", pos, true)
        if pos and pos < where then
            line = line + 1
            linepos = pos
            pos = pos + 1
        else
            break
        end
    end
    return "line " .. line .. ", column " .. (where - linepos)
end

local function unterminated (str, what, where)
    return nil, strlen (str) + 1, "unterminated " .. what .. " at " .. loc (str, where)
end

local function scanwhite (str, pos)
    while true do
        pos = strfind (str, "%S", pos)
        if not pos then return nil end
        local sub2 = strsub (str, pos, pos + 1)
        if sub2 == "\239\187" and strsub (str, pos + 2, pos + 2) == "\191" then
            -- UTF-8 Byte Order Mark
            pos = pos + 3
        elseif sub2 == "//" then
            pos = strfind (str, "[\n\r]", pos + 2)
            if not pos then return nil end
        elseif sub2 == "/*" then
            pos = strfind (str, "*/", pos + 2)
            if not pos then return nil end
            pos = pos + 2
        else
            return pos
        end
    end
end

local escapechars = {
    ["\""] = "\"", ["\\"] = "\\", ["/"] = "/", ["b"] = "\b", ["f"] = "\f",
    ["n"] = "\n", ["r"] = "\r", ["t"] = "\t"
}

local function unichar (value)
    if value < 0 then
        return nil
    elseif value <= 0x007f then
        return strchar (value)
    elseif value <= 0x07ff then
        return strchar (0xc0 + floor(value/0x40),
            0x80 + (floor(value) % 0x40))
    elseif value <= 0xffff then
        return strchar (0xe0 + floor(value/0x1000),
            0x80 + (floor(value/0x40) % 0x40),
            0x80 + (floor(value) % 0x40))
    elseif value <= 0x10ffff then
        return strchar (0xf0 + floor(value/0x40000),
            0x80 + (floor(value/0x1000) % 0x40),
            0x80 + (floor(value/0x40) % 0x40),
            0x80 + (floor(value) % 0x40))
    else
        return nil
    end
end

local function scanstring (str, pos)
    local lastpos = pos + 1
    local buffer, n = {}, 0
    while true do
        local nextpos = strfind (str, "[\"\\]", lastpos)
        if not nextpos then
            return unterminated (str, "string", pos)
        end
        if nextpos > lastpos then
            n = n + 1
            buffer[n] = strsub (str, lastpos, nextpos - 1)
        end
        if strsub (str, nextpos, nextpos) == "\"" then
            lastpos = nextpos + 1
            break
        else
            local escchar = strsub (str, nextpos + 1, nextpos + 1)
            local value
            if escchar == "u" then
                value = tonumber (strsub (str, nextpos + 2, nextpos + 5), 16)
                if value then
                    local value2
                    if 0xD800 <= value and value <= 0xDBff then
                        -- we have the high surrogate of UTF-16. Check if there is a
                        -- low surrogate escaped nearby to combine them.
                        if strsub (str, nextpos + 6, nextpos + 7) == "\\u" then
                            value2 = tonumber (strsub (str, nextpos + 8, nextpos + 11), 16)
                            if value2 and 0xDC00 <= value2 and value2 <= 0xDFFF then
                                value = (value - 0xD800)  * 0x400 + (value2 - 0xDC00) + 0x10000
                            else
                                value2 = nil -- in case it was out of range for a low surrogate
                            end
                        end
                    end
                    value = value and unichar (value)
                    if value then
                        if value2 then
                            lastpos = nextpos + 12
                        else
                            lastpos = nextpos + 6
                        end
                    end
                end
            end
            if not value then
                value = escapechars[escchar] or escchar
                lastpos = nextpos + 2
            end
            n = n + 1
            buffer[n] = value
        end
    end
    if n == 1 then
        return buffer[1], lastpos
    elseif n > 1 then
        return concat (buffer), lastpos
    else
        return "", lastpos
    end
end

local scanvalue -- forward declaration

local function scantable (what, closechar, str, startpos, nullval, objectmeta, arraymeta)
    local len = strlen (str)
    local tbl, n = {}, 0
    local pos = startpos + 1
    if what == 'object' then
        setmetatable (tbl, objectmeta)
    else
        setmetatable (tbl, arraymeta)
    end
    while true do
        pos = scanwhite (str, pos)
        if not pos then return unterminated (str, what, startpos) end
        local char = strsub (str, pos, pos)
        if char == closechar then
            -- CASTL modification
            if what == 'array' then
                -- shift to 0-based index
                local tmp = rawget(tbl, 1)
                tremove(tbl, 1)
                rawset(tbl, 0, tmp)
                -- assign length
                rawset(tbl, "length", n)
            end

            return tbl, pos + 1
        end
        local val1, err
        val1, pos, err = scanvalue (str, pos, nullval, objectmeta, arraymeta)
        if err then return nil, pos, err end
        pos = scanwhite (str, pos)
        if not pos then return unterminated (str, what, startpos) end
        char = strsub (str, pos, pos)
        if char == ":" then
            if val1 == nil then
                return nil, pos, "cannot use nil as table index (at " .. loc (str, pos) .. ")"
            end
            pos = scanwhite (str, pos + 1)
            if not pos then return unterminated (str, what, startpos) end
            local val2
            val2, pos, err = scanvalue (str, pos, nullval, objectmeta, arraymeta)
            if err then return nil, pos, err end
            tbl[val1] = val2
            pos = scanwhite (str, pos)
            if not pos then return unterminated (str, what, startpos) end
            char = strsub (str, pos, pos)
        else
            n = n + 1
            tbl[n] = val1
        end
        if char == "," then
            pos = pos + 1
        end
    end
end

scanvalue = function (str, pos, nullval, objectmeta, arraymeta)
    pos = pos or 1
    pos = scanwhite (str, pos)
    if not pos then
        return nil, strlen (str) + 1, "no valid JSON value (reached the end)"
    end
    local char = strsub (str, pos, pos)
    if char == "{" then
        return scantable ('object', "}", str, pos, nullval, objectmeta, arraymeta)
    elseif char == "[" then
        return scantable ('array', "]", str, pos, nullval, objectmeta, arraymeta)
    elseif char == "\"" then
        return scanstring (str, pos)
    else
        local pstart, pend = strfind (str, "^%-?[%d%.]+[eE]?[%+%-]?%d*", pos)
        if pstart then
            local number = str2num (strsub (str, pstart, pend))
            if number then
                return number, pend + 1
            end
        end
        pstart, pend = strfind (str, "^%a%w*", pos)
        if pstart then
            local name = strsub (str, pstart, pend)
            if name == "true" then
                return true, pend + 1
            elseif name == "false" then
                return false, pend + 1
            elseif name == "null" then
                return nullval, pend + 1
            end
        end
        return nil, pos, "no valid JSON value at " .. loc (str, pos)
    end
end

local function optionalmetatables(...)
    if select("#", ...) > 0 then
        return ...
    else
        return {__jsontype = 'object'}, {__jsontype = 'array'}
    end
end

function json.decode (str, pos, nullval, ...)
    local objectmeta, arraymeta = optionalmetatables(...)
    return scanvalue (str, pos, nullval, objectmeta, arraymeta)
end

function json.use_lpeg ()
    local g = require ("lpeg")

    if g.version() == "0.11" then
        error "due to a bug in LPeg 0.11, it cannot be used for JSON matching"
    end

    local pegmatch = g.match
    local P, S, R = g.P, g.S, g.R

    local function ErrorCall (str, pos, msg, state)
        if not state.msg then
            state.msg = msg .. " at " .. loc (str, pos)
            state.pos = pos
        end
        return false
    end

    local function Err (msg)
        return g.Cmt (g.Cc (msg) * g.Carg (2), ErrorCall)
    end

    local SingleLineComment = P"//" * (1 - S"\n\r")^0
    local MultiLineComment = P"/*" * (1 - P"*/")^0 * P"*/"
    local Space = (S" \n\r\t" + P"\239\187\191" + SingleLineComment + MultiLineComment)^0

    local PlainChar = 1 - S"\"\\\n\r"
    local EscapeSequence = (P"\\" * g.C (S"\"\\/bfnrt" + Err "unsupported escape sequence")) / escapechars
    local HexDigit = R("09", "af", "AF")
    local function UTF16Surrogate (match, pos, high, low)
        high, low = tonumber (high, 16), tonumber (low, 16)
        if 0xD800 <= high and high <= 0xDBff and 0xDC00 <= low and low <= 0xDFFF then
            return true, unichar ((high - 0xD800)  * 0x400 + (low - 0xDC00) + 0x10000)
        else
            return false
        end
    end
    local function UTF16BMP (hex)
        return unichar (tonumber (hex, 16))
    end
    local U16Sequence = (P"\\u" * g.C (HexDigit * HexDigit * HexDigit * HexDigit))
    local UnicodeEscape = g.Cmt (U16Sequence * U16Sequence, UTF16Surrogate) + U16Sequence/UTF16BMP
    local Char = UnicodeEscape + EscapeSequence + PlainChar
    local String = P"\"" * g.Cs (Char ^ 0) * (P"\"" + Err "unterminated string")
    local Integer = P"-"^(-1) * (P"0" + (R"19" * R"09"^0))
    local Fractal = P"." * R"09"^0
    local Exponent = (S"eE") * (S"+-")^(-1) * R"09"^1
    local Number = (Integer * Fractal^(-1) * Exponent^(-1))/str2num
    local Constant = P"true" * g.Cc (true) + P"false" * g.Cc (false) + P"null" * g.Carg (1)
    local SimpleValue = Number + String + Constant
    local ArrayContent, ObjectContent

    -- The functions parsearray and parseobject parse only a single value/pair
    -- at a time and store them directly to avoid hitting the LPeg limits.
    local function parsearray (str, pos, nullval, state)
        local obj, cont
        local npos
        local t, nt = {}, 0
        repeat
            obj, cont, npos = pegmatch (ArrayContent, str, pos, nullval, state)
            if not npos then break end
            pos = npos
            nt = nt + 1
            t[nt] = obj
        until cont == 'last'
        return pos, setmetatable (t, state.arraymeta)
    end

    local function parseobject (str, pos, nullval, state)
        local obj, key, cont
        local npos
        local t = {}
        repeat
            key, obj, cont, npos = pegmatch (ObjectContent, str, pos, nullval, state)
            if not npos then break end
            pos = npos
            t[key] = obj
        until cont == 'last'
        return pos, setmetatable (t, state.objectmeta)
    end

    local Array = P"[" * g.Cmt (g.Carg(1) * g.Carg(2), parsearray) * Space * (P"]" + Err "']' expected")
    local Object = P"{" * g.Cmt (g.Carg(1) * g.Carg(2), parseobject) * Space * (P"}" + Err "'}' expected")
    local Value = Space * (Array + Object + SimpleValue)
    local ExpectedValue = Value + Space * Err "value expected"
    ArrayContent = Value * Space * (P"," * g.Cc'cont' + g.Cc'last') * g.Cp()
    local Pair = g.Cg (Space * String * Space * (P":" + Err "colon expected") * ExpectedValue)
    ObjectContent = Pair * Space * (P"," * g.Cc'cont' + g.Cc'last') * g.Cp()
    local DecodeValue = ExpectedValue * g.Cp ()

    function json.decode (str, pos, nullval, ...)
        local state = {}
        state.objectmeta, state.arraymeta = optionalmetatables(...)
        local obj, retpos = pegmatch (DecodeValue, str, pos, nullval, state)
        if state.msg then
            return nil, state.pos, state.msg
        else
            return obj, retpos
        end
    end

    -- use this function only once:
    json.use_lpeg = function () return json end

    json.using_lpeg = true

    return json -- so you can get the module using json = require "dkjson".use_lpeg()
end

if always_try_using_lpeg then
    pcall (json.use_lpeg)
end

return json

end
end

do
local _ENV = _ENV
package.preload[ "castl.modules.error_helper" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local errorHelper = {}

local new
local Error, TypeError, SyntaxError, RangeError, ReferenceError, URIError, EvalError
local require = require

_ENV = nil

local loadError = function()
    new = new or require("castl.core_objects").new
    Error = Error or require("castl.constructor.error.error")
end
local loadTypeError = function()
    new = new or require("castl.core_objects").new
    TypeError = TypeError or require("castl.constructor.error.type_error")
end
local loadSyntaxError = function()
    new = new or require("castl.core_objects").new
    SyntaxError = SyntaxError or require("castl.constructor.error.syntax_error")
end
local loadRangeError = function()
    new = new or require("castl.core_objects").new
    RangeError = RangeError or require("castl.constructor.error.range_error")
end
local loadReferenceError = function()
    new = new or require("castl.core_objects").new
    ReferenceError = ReferenceError or require("castl.constructor.error.reference_error")
end
local loadEvalError = function()
    new = new or require("castl.core_objects").new
    EvalError = EvalError or require("castl.constructor.error.eval_error")
end
local loadURIError = function()
    new = new or require("castl.core_objects").new
    URIError = URIError or require("castl.constructor.error.uri_error")
end

function errorHelper.newError(message)
    loadError()
    return new(Error, message)
end
function errorHelper.newTypeError(message)
    loadTypeError()
    return new(TypeError, message)
end
function errorHelper.newSyntaxError(message)
    loadSyntaxError()
    return new(SyntaxError, message)
end
function errorHelper.newRangeError(message)
    loadRangeError()
    return new(RangeError, message)
end
function errorHelper.newReferenceError(message)
    loadReferenceError()
    return new(ReferenceError, message)
end
function errorHelper.newURIError(message)
    loadURIError()
    return new(URIError, message)
end
function errorHelper.newEvalError(message)
    loadEvalError()
    return new(EvalError, message)
end

return errorHelper

end
end

do
local _ENV = _ENV
package.preload[ "castl.modules.regexphelper" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local regexHelper = {}

local rex
local require, type = require, type
local regexEngineName = "rex_pcre"

_ENV = nil

regexHelper.getRex = function()
    return rex or require(regexEngineName)
end

regexHelper.getPCRECompilationFlag = function(regexp)
    if regexp.ignoreCase and regexp.multiline then
        return "im"
    elseif regexp.ignoreCase then
        return "i"
    elseif regexp.multiline then
        return "m"
    end

    return nil
end

regexHelper.split = function(subject, regexp)
    rex = rex or require(regexEngineName)

    -- compilation flags
    local cf = regexHelper.getPCRECompilationFlag(regexp)

    return rex.split(subject, regexp.source, cf)
end

-- TODO: it's a hack. maybe I missed something but it's the only way
-- I found to know if second returned argument of rex.split is a capture or only the separator pattern itself
-- http://rrthomas.github.io/lrexlib/manual.html#split
regexHelper.regExpHasCaptures = function(subject, regexp)
    rex = rex or require(regexEngineName)
    local _,_,capture = rex.find(subject, regexp.source)
    return capture
end

regexHelper.gsub = function(subject, regexp, replacer)
    rex = rex or require(regexEngineName)

    -- compilation flags
    local cf = regexHelper.getPCRECompilationFlag(regexp)
    local n = not regexp.global and 1 or nil

    local match = regexp.source

    -- add a global capture to get the entire match
    -- passed to the function
    if type(replacer) == "function" then
        match = "(" .. match .. ")"
    end

    return rex.gsub(subject, match, replacer, n, cf)
end

return regexHelper

end
end

do
local _ENV = _ENV
package.preload[ "castl.others" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL others submodule]] --

local coreObjects = require("castl.core_objects")

local others = {}
local gsub = string.gsub
local print, type = print, type

_ENV = nil

others.console = coreObjects.obj({
    log = function (self, ...)
        print(...)
    end,
    info = function (self, ...)
        print(...)
    end,
    warn = function (self, ...)
        print(...)
    end,
    error = function (self, ...)
        print(...)
    end,
    dir = function(self, o)
        if type(o) == "string" then
            o = gsub(o, "\\", "\\\\")
            o = gsub(o, "\n", "\\n")
            o = gsub(o, "\r", "\\r")
            o = gsub(o, "\t", "\\t")
            o = gsub(o, "\b", "\\b")
            o = gsub(o, "\f", "\\f")
            o = "'" .. o .. "'";
        end
        print(o)
    end
})

return others

end
end

do
local _ENV = _ENV
package.preload[ "castl.protos" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local protos = {}

-- forward declaration
protos.arrayProto = {}
protos.booleanProto = {}
protos.dateProto = {}
protos.functionProto = {}
protos.numberProto = {}
protos.objectProto = {}
protos.regexpProto = {}
protos.stringProto = {}

protos.errorProto = {}
protos.rangeErrorProto = {}
protos.referenceErrorProto = {}
protos.syntaxErrorProto = {}
protos.typeErrorProto = {}
protos.evalErrorProto = {}
protos.uriErrorProto = {}
protos.callSiteProto = {}

-- load definition
protos.loadPrototypesDefinition = function()
    require("castl.prototype.array")(protos.arrayProto)
    require("castl.prototype.boolean")(protos.booleanProto)
    require("castl.prototype.date")(protos.dateProto)
    require("castl.prototype.function")(protos.functionProto)
    require("castl.prototype.number")(protos.numberProto)
    require("castl.prototype.object")(protos.objectProto)
    require("castl.prototype.regexp")(protos.regexpProto)
    require("castl.prototype.string")(protos.stringProto)

    require("castl.prototype.error.error")(protos.errorProto)
    require("castl.prototype.error.range_error")(protos.rangeErrorProto)
    require("castl.prototype.error.reference_error")(protos.referenceErrorProto)
    require("castl.prototype.error.syntax_error")(protos.syntaxErrorProto)
    require("castl.prototype.error.type_error")(protos.typeErrorProto)
    require("castl.prototype.error.eval_error")(protos.evalErrorProto)
    require("castl.prototype.error.uri_error")(protos.uriErrorProto)
    require("castl.prototype.error.call_site")(protos.callSiteProto)
end

return protos

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.array" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Array prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/prototype

return function(arrayPrototype)
    local runtime
    local makeArray = require("castl.core_objects").array
    local boolean = require("castl.jssupport").boolean
    local internal = require("castl.internal")
    local errorHelper = require("castl.modules.error_helper")
    local throw = require("castl.jssupport").throw

    local rawget, rawset, require, getmetatable = rawget, rawset, require, getmetatable
    local tonumber, min, floor, type = tonumber, math.min, math.floor, type
    local pack = table.pack or function(...) return {n = select('#',...),...} end
    local remove, insert, sort = table.remove, table.insert, table.sort
    local null, ToString, ToInteger, ToNumber = internal.null, internal.ToString, internal.ToInteger, internal.ToNumber

    _ENV = nil

    arrayPrototype.length = 0

    local negativeIndex = function (index, length)
        local k = length + index
        if k < 0 then
            return 0
        end
        return k
    end

    arrayPrototype.toString = function (this)
        return arrayPrototype.join(this)
    end

    arrayPrototype.toLocaleString = arrayPrototype.toString

    arrayPrototype.push = function (this, ...)
        local args = pack(...)
        local length = this.length

        for i = 1, args.n do
            rawset(this, length, args[i])
            length = length + 1
        end

        rawset(this, "length", length)

        return length
    end

    arrayPrototype.pop = function (this)
        local length = this.length
        if length == 0 then
            return nil
        end

        local value = rawget(this, length - 1)
        rawset(this, length - 1, nil)
        rawset(this, "length", length - 1)
        return value
    end

    arrayPrototype.shift = function (this)
        local length = this.length

        if length > 0 then
            local value = rawget(this, 0)
            rawset(this, 0, remove(this, 1) or nil)
            rawset(this, 'length', length - 1)
            return value
        end

        return nil
    end

    arrayPrototype.unshift = function (this, ...)
        local args = pack(...)
        local newLength = this["length"] + args.n

        for i = args.n, 1, -1 do
            insert(this, 1, rawget(this, 0))
            rawset(this, 0, args[i])
        end

        rawset(this, 'length', newLength)

        return newLength
    end

    arrayPrototype.splice = function (this, index, howMany, ...)
        local elements = pack(...)
        local length = this.length

        if index > length then
            index = length
        elseif index < 0 then
            index = negativeIndex(index, length)
        end

        howMany = min(tonumber(howMany) or (length - index), length - index)

        -- delete elements and collect deleted elements
        local ret = {}
        for i = 1,howMany do
            ret[i - 1] = rawget(this, index)
            if index == 0 then
                rawset(this, 0, remove(this, 1) or nil)
            else
                remove(this, index)
            end
        end

        -- insert new elements
        for i = 1,elements.n do
            if index == 0 then
                insert(this, 1, rawget(this, 0))
                rawset(this, 0, elements[i])
            else
                insert(this, index, elements[i])
            end

            index = index + 1
        end

        -- new length
        rawset(this, 'length', length - howMany + elements.n)

        -- return array object
        return makeArray(ret, howMany)
    end

    arrayPrototype.reverse = function (this)
        local length = this.length

        for i = 0, floor(length / 2) - 1 do
            -- swap
            local tmp = this[i]
            this[i] = this[length - 1 - i]
            this[length - 1 - i] = tmp
        end
        return this
    end

    arrayPrototype.slice = function (this, beginSlice, endSlice)
        local length = this.length
        beginSlice = beginSlice or 0
        endSlice = endSlice or length

        if beginSlice < 0 then
            beginSlice = negativeIndex(beginSlice, length)
        end
        if endSlice < 0 then
            endSlice = negativeIndex(endSlice, length)
        elseif endSlice > length then
            endSlice = length
        end

        local ret, j = {}, 0
        for i = beginSlice, endSlice - 1 do
            ret[j] = this[i]
            j = j + 1
        end

        return makeArray(ret, j)
    end

    arrayPrototype.concat = function (this, ...)
        local ret = {}
        local length
        local args = pack(...)

        -- if this is a native Array
        if (getmetatable(this) or {})._prototype == arrayPrototype then
            for i = 1, this.length do
                ret[i] = this[i - 1]
            end
            length = this.length
        else
            ret[1] = this
            length = 1
        end

        for i = 1, args.n do
            -- test if is array
            if (getmetatable(args[i]) or {})._prototype == arrayPrototype then
                for j = 0, args[i].length -1 do
                    insert(ret, args[i][j])
                end
                length = length + args[i].length
            else
                insert(ret, args[i])
                length = length + 1
            end
        end

        -- shift to 0-based index
        local tmp = rawget(ret, 1)
        remove(ret, 1)
        rawset(ret, 0, tmp)

        -- convert to array object
        return makeArray(ret, length)
    end

    -- doesn't work if two nil follow in the array
    arrayPrototype.sort = function (this, compareFunction)
        -- shift from 0-based to 1-based index
        insert(this, 1, this[0])
        rawset(this, 0, nil)

        -- sort
        sort(this, function (a, b)
            if b == nil and a ~= nil then
                return true
            end
            if a == nil and b ~= nil then
                return false
            end

            if not compareFunction then
                return (ToString(a) < ToString(b))
            else
                local comp = compareFunction(this, a, b)
                local tcomp = type(comp)
                if tcomp == "number" then
                    return comp < 0
                elseif tcomp == "boolean" then
                    if a == b then
                        return false
                    end
                    return not comp
                end
            end
        end
        )

        --  unshift
        local tmp = rawget(this, 1)
        remove(this, 1)
        rawset(this, 0, tmp)

        return this
    end

    arrayPrototype.join = function (this, arg)
        local separator = ","
        if arg ~= nil then
            separator = ToString(arg)
        end

        local str = ""
        local bound = this.length - 1

        for i = 0, bound do
            if not (this[i] == nil or this[i] == null) then
                str = str .. ToString(this[i])
            end
            if i ~= bound then
                str = str .. separator
            end
        end

        return str
    end

    arrayPrototype.lastIndexOf = function(this, searchElement, fromIndex)
        local length = this.length
        local n = fromIndex ~= nil and ToInteger(fromIndex) or length - 1

        if n < 0 then
            n = length + n
        elseif n > length then
            n = length - 1
        end

        for i = n, 0, -1 do
            if this[i] == searchElement then
                return i
            end
        end

        return -1
    end

    arrayPrototype.indexOf = function (this, searchElement, fromIndex)
        local length = this.length
        local n = fromIndex ~= nil and ToNumber(fromIndex) or 0

        if n >= length then
            return -1
        end
        if n < 0 then
            n = negativeIndex(n, length)
        end

        for i = n, length - 1 do
            if this[i] == searchElement then
                return i
            end
        end

        return -1
    end

    local getThisArg = function(arg)
        -- if thisArg's not defined, use default this
        if arg == nil then
            runtime = runtime or require("castl.runtime")
            return runtime
        end

        return arg
    end

    arrayPrototype.map = function (this, callback, thisArg)
        local ret = {}

        thisArg = getThisArg(thisArg)

        for i = 0, this.length - 1 do
            if this[i] ~= nil then
                local v = callback(thisArg, this[i], i, this)
                insert(ret, v)
            end
        end

        -- shift to 0-based index
        local tmp = rawget(ret, 1)
        remove(ret, 1)
        rawset(ret, 0, tmp)

        -- convert to array object
        return makeArray(ret, this.length)
    end

    arrayPrototype.filter = function (this, callback, thisArg)
        local ret, length = {}, 0

        thisArg = getThisArg(thisArg)

        for i = 0, this.length - 1 do
            -- filter
            if this[i] ~= nil and boolean(callback(thisArg, this[i], i, this)) then
                insert(ret, this[i])
                length = length + 1
            end
        end

        -- shift to 0-based index
        local tmp = rawget(ret, 1)
        remove(ret, 1)
        rawset(ret, 0, tmp)

        -- convert to array object
        return makeArray(ret, length)
    end

    local empty = function(array)
        return array.length == 0 or array:every(function(_,e) return e == nil end)
    end

    arrayPrototype.reduce = function (this, callback, initialValue)
        if empty(this) and initialValue == nil then
            throw(errorHelper.newTypeError("Reduce of empty array with no initial value"))
        end

        local value = initialValue or this[0]
        local start = initialValue and 0 or 1
        local thisArg = getThisArg()
        local length = this.length
        for i = start, length - 1 do
            -- if not nil (else ignore)
            if this[i] ~= nil then
                value = callback(thisArg, value, this[i], i, this)
            end
        end

        return value
    end

    arrayPrototype.reduceRight = function (this, callback, initialValue)
        if empty(this) and initialValue == nil then
            throw(errorHelper.newTypeError("Reduce of empty array with no initial value"))
        end

        local value = initialValue or this[this.length - 1]
        local start = initialValue and this.length - 1 or this.length - 2
        local thisArg = getThisArg()
        for i = start, 0, -1 do
            -- if not nil (else ignore)
            if this[i] ~= nil then
                value = callback(thisArg, value, this[i], i, this)
            end
        end

        return value
    end

    arrayPrototype.forEach = function (this, callback, thisArg)
        thisArg = getThisArg(thisArg)

        local tthis = type(this)
        if tthis == "table" or tthis == "string" then
            local bound = this.length - 1
            for i = 0, bound do
                if this[i] ~= nil then
                    callback(thisArg, this[i], i, this)
                end
            end
        end
    end

    arrayPrototype.some = function (this, callback, thisArg)
        thisArg = getThisArg(thisArg)

        for i = 0, this.length - 1 do
            if this[i] ~= nil and boolean(callback(thisArg, this[i], i, this)) then
                return true
            end
        end

        return false
    end

    arrayPrototype.every = function (this, callback, thisArg)
        thisArg = getThisArg(thisArg)

        for i = 0, this.length - 1 do
            if not boolean(callback(thisArg, this[i], i, this)) then
                return false
            end
        end

        return true
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.boolean" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Boolean prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Boolean/prototype

return function(booleanPrototype)
    local getmetatable, tostring, type = getmetatable, tostring, type

    _ENV = nil

    booleanPrototype.toString = function (this)
        return tostring(this:valueOf())
    end

    booleanPrototype.valueOf = function (this)
        if type(this) == "boolean" then
            return this
        else
            return getmetatable(this)._primitive
        end
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.date" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Date prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/prototype

return function(datePrototype)
    local date, time, difftime = os.date, os.time, os.difftime
    local format = string.format

    _ENV = nil

    datePrototype.toString = function (this)
        return date('%a %h %d %Y %H:%M:%S GMT%z (%Z)', this._timestamp / 1000)
    end

    datePrototype.getDate = function (this) return date('*t', this._timestamp / 1000).day end
    datePrototype.getDay = function (this) return date('*t', this._timestamp / 1000).wday - 1 end
    datePrototype.getFullYear = function (this) return date('*t', this._timestamp / 1000).year end
    datePrototype.getHours = function (this) return date('*t', this._timestamp / 1000).hour end
    datePrototype.getMilliseconds = function (this) return this._timestamp % 1000 end
    datePrototype.getMinutes = function (this) return date('*t', this._timestamp / 1000).min end
    datePrototype.getMonth = function (this) return date('*t', this._timestamp / 1000).month - 1 end
    datePrototype.getSeconds = function (this) return date('*t', this._timestamp / 1000).sec end

    datePrototype.getTime = function(this)
        return this._timestamp
    end

    datePrototype.getTimezoneOffset = function(this)
        local now = time()
        return difftime(now, time(date("!*t", now))) / 60
    end

    datePrototype.getUTCDate = function (this) return date('!*t', this._timestamp / 1000).day end
    datePrototype.getUTCDay = function (this) return date('!*t', this._timestamp / 1000).wday - 1 end
    datePrototype.getUTCFullYear = function (this) return date('!*t', this._timestamp / 1000).year end
    datePrototype.getUTCHours = function (this) return date('!*t', this._timestamp / 1000).hour end
    datePrototype.getUTCMilliseconds = datePrototype.getMilliseconds
    datePrototype.getUTCMinutes = function (this) return date('!*t', this._timestamp / 1000).min end
    datePrototype.getUTCMonth = function (this) return date('!*t', this._timestamp / 1000).month - 1 end
    datePrototype.getUTCSeconds = function (this) return date('!*t', this._timestamp / 1000).sec end

    local setDateTime = function(this, key, value)
        local millis = this._timestamp % 1000
        local d = date('*t', this._timestamp / 1000)
        d[key] = value
        this._timestamp = (time(d) * 1000) + millis
    end

    local setDateTimeUTC = function(this, key, value)
        local millis = this._timestamp % 1000
        local d = date('!*t', this._timestamp / 1000)
        d[key] = value
        this._timestamp = (time(d) * 1000) + millis
    end

    datePrototype.setDate = function (this, dayValue) setDateTime(this, "day", dayValue) end
    datePrototype.setFullYear = function (this, yearValue) setDateTime(this, "year", yearValue) end
    datePrototype.setHours = function (this, hoursValue) setDateTime(this, "hour", hoursValue) end
    datePrototype.setMinutes = function (this, minutesValue) setDateTime(this, "min", minutesValue) end
    datePrototype.setMonth = function (this, monthValue) setDateTime(this, "month", monthValue + 1) end
    datePrototype.setSeconds = function (this, secondsValue) setDateTime(this, "sec", secondsValue) end

    datePrototype.setMilliseconds = function (this, millisValue)
        local d = date('*t', this._timestamp / 1000)
        this._timestamp = (time(d) * 1000)  + millisValue
    end

    datePrototype.setTime = function (this, timeValue)
        this._timestamp = timeValue
    end

    datePrototype.setUTCDate = function (this, dayValue) setDateTimeUTC(this, "day", dayValue) end
    datePrototype.setUTCFullYear = function (this, yearValue) setDateTimeUTC(this, "year", yearValue) end
    datePrototype.setUTCHours = function (this, hoursValue) setDateTimeUTC(this, "hour", hoursValue) end
    datePrototype.setUTCMinutes = function (this, minutesValue) setDateTimeUTC(this, "min", minutesValue) end
    datePrototype.setUTCMonth = function (this, monthValue) setDateTimeUTC(this, "month", monthValue + 1) end
    datePrototype.setUTCSeconds = function (this, secondsValue) setDateTimeUTC(this, "sec", secondsValue) end

    datePrototype.setUTCMilliseconds = function (this, millisValue)
        local d = date('!*t', this._timestamp / 1000)
        this._timestamp = (time(d) * 1000) + millisValue
    end

    datePrototype.toDateString = function(this)
        return date('%a %h %d %Y', this._timestamp / 1000)
    end

    datePrototype.toISOString = function(this)
        return date('!%Y-%m-%dT%H:%M:%S.', this._timestamp / 1000) .. format('%03dZ', this._timestamp % 1000)
    end

    datePrototype.toJSON = datePrototype.toISOString
    datePrototype.toLocaleDateString = datePrototype.toDateString
    datePrototype.toLocaleString = datePrototype.toString

    datePrototype.toTimeString = function(this)
        return date('%H:%M:%S GMT%z (%Z)', this._timestamp / 1000)
    end
    
    datePrototype.toLocaleTimeString = function(this)
        return date('%H:%M:%S', this._timestamp / 1000)
    end

    datePrototype.toUTCString = function(this)
        return date('!%a %h %d %Y %H:%M:%S GMT', this._timestamp / 1000)
    end

    datePrototype.valueOf = function(this)
        return this._timestamp
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.call_site" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL CallSite prototype submodule]] --

return function(callSitePrototype)
    local getmetatable = getmetatable
    local gsub = string.gsub
    _ENV = nil

    callSitePrototype.getThis = function(this)
        return this.receiver
    end
    callSitePrototype.getTypeName = function(this)
    end
    callSitePrototype.getFunction = function(this)
        return this.fun
    end
    callSitePrototype.getFunctionName = function(this)
    end
    callSitePrototype.getMethodName = function(this)
    end
    callSitePrototype.getFileName = function(this)
        return getmetatable(this).frame.short_src
    end
    callSitePrototype.getLineNumber = function(this)
        return getmetatable(this).frame.currentline
    end
    callSitePrototype.getColumnNumber = function(this)
    end
    callSitePrototype.getEvalOrigin = function(this)
    end
    callSitePrototype.isToplevel = function(this)
    end
    callSitePrototype.isEval = function(this)
    end
    callSitePrototype.isNative = function(this)
    end
    callSitePrototype.isConstructor = function(this)
    end

    callSitePrototype.toString = function (this)
        local frame = getmetatable(this).frame
        local name = frame.name or "<anonymous>"
        local file = gsub(frame.short_src, "%[[TC]%]. ", '')
        return name .. " (" .. file .. ":" .. frame.currentline .. ")"
    end
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Error prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Error/prototype

return function(errorPrototype)
    _ENV = nil

    errorPrototype.name = "Error"
    errorPrototype.message = ""

    errorPrototype.toString = function (this)
        if this.message ~= "" then
            return this.name .. ": " .. this.message
        else
            return this.name
        end
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.eval_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL URIError prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/URIError/prototype

return function(evalErrorPrototype)
    _ENV = nil

    evalErrorPrototype.name = "EvalError"
    evalErrorPrototype.message = ""
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.range_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL RangeError prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RangeError/prototype

return function(rangeErrorPrototype)
    _ENV = nil

    rangeErrorPrototype.name = "RangeError"
    rangeErrorPrototype.message = ""
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.reference_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL ReferenceError prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/ReferenceError/prototype

return function(referenceErrorPrototype)
    _ENV = nil

    referenceErrorPrototype.name = "ReferenceError"
    referenceErrorPrototype.message = ""
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.syntax_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL SyntaxError prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/SyntaxError/prototype

return function(syntaxErrorPrototype)
    _ENV = nil

    syntaxErrorPrototype.name = "SyntaxError"
    syntaxErrorPrototype.message = ""
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.type_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL TypeError prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/TypeError/prototype

return function(typeErrorPrototype)
    _ENV = nil

    typeErrorPrototype.name = "TypeError"
    typeErrorPrototype.message = ""
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.error.uri_error" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL URIError prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/URIError/prototype

return function(uriErrorPrototype)
    _ENV = nil

    uriErrorPrototype.name = "URIError"
    uriErrorPrototype.message = ""
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.function" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Function prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/prototype

return function(functionPrototype)
    local pack = table.pack or function(...) return {n = select('#',...),...} end
    local unpack = table.unpack or unpack

    _ENV = nil

    functionPrototype.call = function(this, thisArg, ...)
        return this(thisArg, ...)
    end

    functionPrototype.bind = function(this, thisArg, ...)
        local argsToPrepend = pack(...)

        return function(_, ...)
            local argset, j = {}, 1
            local argsToAppend = pack(...)

            for i = 1, argsToPrepend.n do
                argset[j] = argsToPrepend[i]
                j = j + 1
            end

            for i = 1, argsToAppend.n do
                argset[j] = argsToAppend[i]
                j = j + 1
            end

            return this(thisArg, unpack(argset, 1, argsToPrepend.n + argsToAppend.n))
        end
    end

    functionPrototype.apply = function(this, thisArg, argsArray)
        local args, length = {}, 0

        if argsArray ~= nil then
            length = argsArray.length
            for i = 0, length - 1 do
                args[i+1] = argsArray[i]
            end
        end

        return this(thisArg, unpack(args, 1, length))
    end

    functionPrototype.toString = function(this)
        return "function (){}"
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.number" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Number prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/prototype

return function(numberPrototype)
    local errorHelper = require("castl.modules.error_helper")
    local throw = require("castl.jssupport").throw

    local tonumber, tostring, floor, concat, insert = tonumber, tostring, math.floor, table.concat, table.insert
    local strsub, strlen, gsub, format, getmetatable, type = string.sub, string.len, string.gsub, string.format, getmetatable, type

    _ENV = nil

    numberPrototype.valueOf = function (this)
        if type(this) == "number" then
            return this
        else
            return getmetatable(this)._primitive
        end
    end

    local valueof = numberPrototype.valueOf

    numberPrototype.toString = function(this, radix)
        local value = valueof(this)

        if not radix or radix == 10 then
            return tostring(value)
        end

        -- TODO: do not handle floating point numbers
        -- http://stackoverflow.com/a/3554821
        local n = floor(value)
        local digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        local t = {}
        local sign = ""
        if n < 0 then
            sign = "-"
            n = -n
        end
        repeat
            local d = (n % radix) + 1
            n = floor(n / radix)
            insert(t, 1, strsub(digits, d, d))
        until n == 0

        return sign .. concat(t, "")
    end

    numberPrototype.toLocaleString = numberPrototype.toString

    numberPrototype.toFixed = function(this, digits)
        local value = valueof(this)
        digits = digits or 0
        return format("%." .. tonumber(digits) .. "f", value)
    end

    numberPrototype.toExponential = function(this, fractionDigits)
        local value = valueof(this)
        if fractionDigits == nil then
            fractionDigits = strlen(tostring(value)) - 1
            if floor(value) ~= value then
                fractionDigits = fractionDigits - 1
            end
        end
        if fractionDigits < 0 or fractionDigits > 20 then
            throw(errorHelper.newRangeError("toExponential() argument must be between 0 and 20"))
        end
        local formatted = format("%." .. fractionDigits .. "e", value)
        return (gsub(formatted, "%+0", "+"))
    end

    numberPrototype.toPrecision = function(this, precision)
        local value = valueof(this)
        if precision == nil then return tostring(value) end
        if precision < 1 or precision > 21 then
            throw(errorHelper.newRangeError("toPrecision() argument must be between 1 and 21"))
        end
        local formatted = format("%." .. precision .. "g", value)
        return (gsub(formatted, "%+0", "+"))
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.object" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL Object prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/prototype

return function(objectPrototype)
    local internal = require("castl.internal")
    local protos = require("castl.protos")
    local arrayProto = protos.arrayProto
    local regexpProto = protos.regexpProto
    local dateProto = protos.dateProto

    local type, rawget = type, rawget
    local getmetatable = getmetatable
    local null, getPrototype, getFunctionProxy = internal.null, internal.prototype, internal.getFunctionProxy

    _ENV = nil

    objectPrototype.toString = function (this)
        local mt = getmetatable(this)
        local tthis = type(this)
        if this == nil then
            return "[object Undefined]"
        elseif this == null then
            return "[object Null]"
        elseif tthis == "string" then
            return "[object String]"
        elseif tthis == "number" then
            return "[object Number]"
        elseif tthis == "boolean" then
            return "[object Boolean]"
        elseif mt and mt._prototype == arrayProto then
            return "[object Array]"
        elseif mt and mt._prototype == regexpProto then
            return "[object RegExp]"
        elseif mt and mt._prototype == dateProto then
            return "[object Date]"
        elseif mt and mt._arguments then
            return "[object Arguments]"
        elseif tthis == "function" then
            return "[object Function]"
        end

        return '[object Object]'
    end

    objectPrototype.toLocaleString = objectPrototype.toString

    objectPrototype.valueOf = function (this)
        return this
    end

    objectPrototype.hasOwnProperty = function (this, p)
        local tthis = type(this)
        if tthis == "string" then
            return p == "length"
        end
        if tthis == "number" then
            return false
        end
        if tthis == "boolean" then
            return false
        end
        if tthis == "function" then
            this = getFunctionProxy(this)
        end

        return rawget(this, p) ~= nil
    end

    objectPrototype.isPrototypeOf = function(this, object)
        if this ~= nil then
            local classPrototypeAttribute = this
            local objProto = getPrototype(object)

            while objProto do
                if objProto == classPrototypeAttribute then
                    return true
                end
                objProto = getPrototype(objProto)
            end
        end
        return false
    end

    -- TODO: enumerability
    objectPrototype.propertyIsEnumerable = function(this, prop)
        return this[prop] ~= nil
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.regexp" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL RegExp prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RegExp/prototype

return function(regexpPrototype)
    local array = require("castl.core_objects").array
    local internal = require("castl.internal")
    local regexpHelper = require("castl.modules.regexphelper")

    local rawget, rawset = rawget, rawset
    local pack = table.pack or function(...) return {n = select('#',...),...} end
    local tinsert, tremove = table.insert, table.remove
    local null, ToString = internal.null, internal.ToString

    _ENV = nil

    regexpPrototype.global = false
    regexpPrototype.ignoreCase = false
    regexpPrototype.multiline = false
    regexpPrototype.source = ""
    regexpPrototype.lastIndex = 0

    regexpPrototype.exec = function(this, str)
        str = ToString(str)
        -- add a global capture to get the entire match
        -- (find don't return the entire match, only  the captures)
        local source =  "(" .. this.source .. ")"

        local cf = regexpHelper.getPCRECompilationFlag(this)
        local ret = {}

        local rex = regexpHelper.getRex()
        if this.global then
            local found = pack(rex.find(str, source, this.lastIndex + 1, cf))
            if found[1] then
                this.lastIndex = found[2]
                ret.index = found[1] - 1
                ret.input = str
                for i = 3, found.n do
                    tinsert(ret, found[i])
                end
            else
                this.lastIndex = 0
            end
        else
            local found = pack(rex.find(str, source, 1, cf))
            if found[1] then
                ret.index = found[1] - 1
                ret.input = str
                for i = 3, found.n do
                    tinsert(ret, found[i])
                end
            end
        end

        local length = #ret

        if length == 0 then
            return null
        end

        -- shift to 0-based index
        local tmp = rawget(ret, 1)
        tremove(ret, 1)
        rawset(ret, 0, tmp)

        return array(ret, length)
    end

    regexpPrototype.toString = function(this)
        local flags = ""
        if rawget(this, "global") then flags = flags .. 'g' end
        if rawget(this, "ignoreCase") then flags = flags .. 'i' end
        if rawget(this, "multiline") then flags = flags .. 'm' end

        return '/' .. this.source .. '/' .. flags
    end

    regexpPrototype.test = function(this, str)
        str = ToString(str)
        local cf = regexpHelper.getPCRECompilationFlag(this)

        local ret = false
        local rex = regexpHelper.getRex()
        if this.global then
            local _, endFound = rex.find(str, this.source, this.lastIndex + 1, cf)
            if endFound then
                this.lastIndex = endFound
                ret = true
            else
                this.lastIndex = 0
                ret = false
            end
        else
            local found = rex.find(str, this.source, 1, cf)
            ret = found and true or false
        end

        return ret
    end
end

end
end

do
local _ENV = _ENV
package.preload[ "castl.prototype.string" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

-- [[ CASTL String prototype submodule]] --
-- https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/prototype

return function(stringPrototype)
    local runtime
    local coreObjects = require("castl.core_objects")
    local _regexp = coreObjects.regexp
    local array = coreObjects.array
    local instanceof = coreObjects.instanceof
    local new = coreObjects.new
    local RegExp = require("castl.constructor.regexp")
    local internal = require("castl.internal")
    local regexpHelper = require("castl.modules.regexphelper")

    local type, tonumber, min, max, rawget, rawset, huge = type, tonumber, math.min, math.max, rawget, rawset, math.huge
    local pack = table.pack or function(...) return {n = select('#',...),...} end
    local tinsert, tremove, concat = table.insert, table.remove, table.concat
    local error, require, getmetatable = error, require, getmetatable
    local sub, byte, gmatch, find, reverse = string.sub, string.byte, string.gmatch, string.find, string.reverse
    local lower, upper, match, gsub, len = string.lower, string.upper, string.match, string.gsub, string.len
    local null, ToString, ToInteger, ToNumber, ToUint32 = internal.null, internal.ToString, internal.ToInteger, internal.ToNumber, internal.ToUint32

    _ENV = nil

    stringPrototype.length = 0

    stringPrototype.valueOf = function (this)
        if type(this) == "string" then
            return this
        end
        return getmetatable(this)._primitive
    end

    local valueof = stringPrototype.valueOf

    local negativeIndex = function (index, length)
        local k = length + index
        if k < 0 then
            return 0
        end
        return k
    end

    stringPrototype.anchor = function(this, name)
        local value = valueof(this)
        local t = {'<a name="'}
        tinsert(t, ToString(name))
        tinsert(t, '">')
        tinsert(t, value)
        tinsert(t, '</a>')
        return concat(t)
    end

    stringPrototype.charAt = function (this, i)
        local value = valueof(this)
        if type(i) == "number" then
            return sub(value, i + 1, i + 1)
        end

        return sub(value, 1, 1)
    end

    stringPrototype.charCodeAt = function (this, i)
        local value = valueof(this)
        i = i or 0
        return byte(value, tonumber(i) + 1)
    end

    stringPrototype.concat = function (this, ...)
        local args = pack(...)
        local ret = valueof(this)

        for i = 1,args.n do
            ret = ret .. ToString(args[i])
        end

        return ret
    end

    stringPrototype.indexOf = function (this, searchValue, fromIndex)
        searchValue = ToString(searchValue)
        local value = valueof(this)
        local length = value.length
        fromIndex = fromIndex ~= nil and ToInteger(fromIndex) or 0
        fromIndex = min(max(fromIndex, 0), length)

        -- special case of empty string
        if searchValue == "" then
            if fromIndex < length then
                return fromIndex
            else
                return length
            end
        end

        -- find
        local ret = find(value, searchValue, fromIndex + 1, true)

        -- not found
        if ret == nil then
            return -1
        end

        return ret - 1
    end

    stringPrototype.lastIndexOf = function (this, searchValue, fromIndex)
        searchValue = ToString(searchValue)
        local value = valueof(this)
        local length = value.length
        fromIndex = fromIndex ~= nil and ToInteger(fromIndex) or huge
        fromIndex = max(fromIndex, 0)

        -- special case of empty string
        if searchValue == "" then
            if fromIndex < length then
                return fromIndex
            else
                return length
            end
        end

        fromIndex = min(fromIndex, length - 1)
        -- find in reversed string
        local start = length - fromIndex - #searchValue + 1
        local locBegin, locEnd = find(reverse(value), reverse(searchValue), start, true)

        -- not found
        if locEnd == nil then
            return -1
        end

        return length - locEnd
    end

    stringPrototype.link = function(this, url)
        local value = valueof(this)
        local t = {'<a href="'}
        tinsert(t, ToString(url))
        tinsert(t, '">')
        tinsert(t, value)
        tinsert(t, '</a>')
        return concat(t)
    end

    stringPrototype.substr = function (this, start, length)
        local value = valueof(this)
        if start < 0 then
            start = negativeIndex(start, value.length)
        end

        if length ~= nil then
            return sub(value, start + 1, start + length)
        else
            return sub(value, start + 1)
        end
    end

    stringPrototype.substring = function (this, indexA, indexB)
        local value = valueof(this)
        if indexA < 0 then
            indexA = 0
        elseif indexA > value.length then
            indexA = value.length
        end

        -- if indexB is specified
        if indexB ~= nil then
            if indexB < 0 then
                indexB = 0
            elseif indexB > value.length then
                indexB = value.length
            end

            -- swap
            if indexA > indexB then
                local tmp = indexB
                indexB = indexA
                indexA = tmp
            end
        end

        if indexB ~= nil then
            return sub(value, indexA + 1, indexB)
        else
            return sub(value, indexA + 1)
        end
    end

    stringPrototype.slice = function (this, beginSlice, endSlice)
        local value = valueof(this)
        if beginSlice < 0 then
            beginSlice = negativeIndex(beginSlice, value.length)
        end

        if endSlice ~= nil then
            if endSlice < 0 then
                endSlice = negativeIndex(endSlice, value.length)
            end

            return sub(value, beginSlice + 1, endSlice)
        end

        return sub(value, beginSlice + 1)

    end

    stringPrototype.small = function(this)
        local value = valueof(this)
        local t = {'<small>'}
        tinsert(t, value)
        tinsert(t, '</small>')
        return concat(t)
    end

    stringPrototype.toLowerCase = function (this)
        return lower(valueof(this))
    end

    stringPrototype.toLocaleLowerCase = function (this)
        return lower(valueof(this))
    end

    stringPrototype.toUpperCase = function (this)
        return upper(valueof(this))
    end

    stringPrototype.toLocaleUpperCase = function (this)
        return upper(valueof(this))
    end

    stringPrototype.toString = function (this)
        local mt = getmetatable(this)
        -- Object String
        if mt and type(mt._primitive) == "string" then
            return mt._primitive
                -- primitive string
        elseif type(this) == 'string' then
            return this
        else
            return "[object Object]"
        end
    end

    stringPrototype.trim = function (this)
        local value = valueof(this)
        -- http://lua-users.org/wiki/StringTrim
        return match(value,'^()%s*$') and '' or match(value,'^%s*(.*%S)')
    end

    --[[
        Functions related to RegExp
    --]]

    stringPrototype.split = function (this, separator, limit)
        local value = valueof(this)
        limit = limit == nil and huge or ToUint32(limit)

        if separator == nil or value == "" then
            return array({[0] = value}, 1)
        end

        local isRegExp = instanceof(separator, RegExp)
        if not isRegExp then
            separator = ToString(separator)
        end

        -- If separator is an empty string, str is converted to an array of characters
        if separator == "" then
            local ret, length = {}, value.length
            for i = 0, length - 1 do
                ret[i] = value:charAt(i)
            end
            return array(ret, length)
        end

        local ret = {}

        if isRegExp then
            local captures = regexpHelper.regExpHasCaptures(value, separator)
            --get iterator
            local iter = regexpHelper.split(value, separator)

            while true do
                local match = pack(iter())
                if not match[1] then break end
                -- insert section
                tinsert(ret, match[1])

                if captures then
                    -- insert captures
                    for i = 2, match.n do
                        tinsert(ret, match[i])
                    end
                end
            end

            local length = #ret
            -- Limit number of results
            for i = length, limit + 1, -1 do
                tremove(ret, i)
            end
        else
            -- separator is a string
            local i = 1
            local start = 1
            local first, last = find(value, separator, start, true)
            while first and i <= limit do
                ret[i] = sub(value, start, first - 1)
                start = last + 1
                first, last = find(value, separator, start, true)
                i = i + 1
            end
            if #ret < limit then
                ret[i] = sub(value, start)
            end
        end

        local length = #ret
        -- shift to 0-based index
        local tmp = rawget(ret, 1)
        tremove(ret, 1)
        rawset(ret, 0, tmp)

        return array(ret, length)
    end

    local getReplacerRegExp = function(newSubStr)
        local replacer = ""

        if type(newSubStr) == "function" then
            runtime = runtime or require("castl.runtime")
            replacer = function(...)
                return ToString(newSubStr(runtime, ...))
            end
        else
            replacer = ToString(newSubStr)
            -- escape %
            replacer = gsub(replacer, "%%", "%%%%")

            -- handle $ parameters
            replacer = gsub(replacer, "%$&", "$0")
            replacer = gsub(replacer, "([%$]+)(.?)",
                function(dollars, follow)
                    local length = len(dollars)
                    local ret = {"%"}
                    for i = 2, length do
                        ret[i] = (i % 2) == 0 and "$" or "%"
                    end

                    if not tonumber(sub(follow, 1, 1)) then
                        ret[length] = "$"
                    end

                    ret[length + 1] = follow

                    return concat(ret)
                end
            )
        end

        return replacer
    end

    local getReplacerString = function(newSubStr)
        local replacer = ToString(newSubStr)

        -- handle $ parameters
        replacer = gsub(replacer, "%$&", "%%0")
        replacer = gsub(replacer, "([%$]+)",
            function(dollars)
                local length = len(dollars)
                local ret = {"$"}
                for i = 2, length do
                    ret[i] = (i % 2) == 1 and "$" or ""
                end

                return concat(ret)
            end
        )

        return replacer
    end

    stringPrototype.replace = function (this, match, newSubStr, flags)
        local value = valueof(this)
        local isRegExp = instanceof(match, RegExp)

        if not isRegExp then
            match = ToString(match)
            -- if flags are passed, match is converted to a regexp
            if flags ~= nil then
                match = new(RegExp, match, flags)
                -- if replacer is a function, match must be converted to a regexp
                -- to be able to use the regexp gsub
            elseif type(newSubStr) == "function" then
                match = new(RegExp, match, "")
            end
        end

        local replacer
        if type(match) == "string" then
            replacer = getReplacerString(newSubStr)
            return (gsub(value, match, replacer, 1))
        else
            replacer = getReplacerRegExp(newSubStr)
            return (regexpHelper.gsub(value, match, replacer))
        end
    end

    stringPrototype.search = function (this, regexp)
        local value = valueof(this)

        if not instanceof(regexp, RegExp) then
            regexp = _regexp(regexp)
        end

        local cf = regexpHelper.getPCRECompilationFlag(regexp)
        local rex = regexpHelper.getRex()
        local found = rex.find(value, regexp.source, 1, cf)

        return found and found - 1 or -1
    end

    stringPrototype.match = function (this, regexp)
        local value = valueof(this)

        if not instanceof(regexp, RegExp) then
            regexp = _regexp(regexp)
        end

        -- add a global capture to get the entire match
        -- (gmatch and find don't return the entire match, only  the captures)
        local source =  "(" .. regexp.source .. ")"

        local cf = regexpHelper.getPCRECompilationFlag(regexp)
        local rex = regexpHelper.getRex()
        local ret = {}

        -- global match: rex.gmatch
        if regexp.global then
            local iter = rex.gmatch(value, source, cf)

            while true do
                local match = pack(iter())
                if not match[1] then break end
                tinsert(ret, match[1])
            end
        else
            -- single match: rex.find (to get the starting index)
            local found = pack(rex.find(value, source, 1, cf))
            if found[1] then
                ret.index = found[1] - 1
                ret.input = value
                for i = 3, found.n do
                    tinsert(ret, found[i])
                end
            end
        end

        local length = #ret

        if length == 0 then
            return null
        end

        -- shift to 0-based index
        local tmp = rawget(ret, 1)
        tremove(ret, 1)
        rawset(ret, 0, tmp)

        return array(ret, length)
    end

end

end
end

do
local _ENV = _ENV
package.preload[ "castl.runtime" ] = function( ... ) local arg = _G.arg;
--[[
    Copyright (c) 2014, Paul Bernier
    
    CASTL is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    CASTL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with CASTL. If not, see <http://www.gnu.org/licenses/>.
--]]

local internal = require("castl.internal")
local protos = require("castl.protos")
local jssupport = require("castl.jssupport")
local coreObjects = require("castl.core_objects")
local globalFunctions = require("castl.global_functions")
local bit = require("castl.modules.bit")
local others = require("castl.others")
local json = require("castl.json")
local Object = require("castl.constructor.object")
local Function = require("castl.constructor.function")
local Array = require("castl.constructor.array")
local Boolean = require("castl.constructor.boolean")
local Number = require("castl.constructor.number")
local String = require("castl.constructor.string")
local Date = require("castl.constructor.date")
local Error = require("castl.constructor.error.error")
local SyntaxError = require("castl.constructor.error.syntax_error")
local TypeError = require("castl.constructor.error.type_error")
local ReferenceError = require("castl.constructor.error.reference_error")
local RangeError = require("castl.constructor.error.range_error")
local URIError = require("castl.constructor.error.uri_error")
local EvalError = require("castl.constructor.error.eval_error")
local RegExp = require("castl.constructor.regexp")
local Math = require("castl.math")
local eval = require("castl.eval")

-- load prototypes definitions
protos.loadPrototypesDefinition()

-- Exported environment
local export = {
    -- jssupport
    _throw = jssupport.throw,
    _eq = jssupport.equal,
    _add = jssupport.add,
    _addStr1 = jssupport.addString1,
    _addStr2 = jssupport.addString2,
    _addNum1 = jssupport.addNumber1,
    _addNum2 = jssupport.addNumber2,
    _bool = jssupport.boolean,
    _type = jssupport.typeof,
    _break = jssupport._break,
    _continue = jssupport._continue,
    _in = jssupport.inOp,
    _with = jssupport.with,
    _void = jssupport.void,
    _e = jssupport.e,
    _inc = jssupport.inc,
    _dec = jssupport.dec,
    _mod = jssupport.modulo,
    _lt = jssupport.lt,
    _le = jssupport.le,
    _gt = jssupport.gt,
    _ge = jssupport.ge,
    NaN = 0/0,
    Infinity = jssupport.Infinity,

    -- global functions
    isNaN = globalFunctions.isNaN,
    isFinite = globalFunctions.isFinite,
    parseFloat = globalFunctions.parseFloat,
    parseInt = globalFunctions.parseInt,
    encodeURI = globalFunctions.encodeURI,
    encodeURIComponent = globalFunctions.encodeURIComponent,
    decodeURI = globalFunctions.decodeURI,
    decodeURIComponent = globalFunctions.decodeURIComponent,
    eval = eval.eval,

    -- internal
    _tonum = internal.ToNumber,
    null = internal.null,

    -- core object
    _obj = coreObjects.obj,
    _arr = coreObjects.array,
    _regexp = coreObjects.regexp,
    _args = coreObjects.arguments,
    _new = coreObjects.new,
    _instanceof = coreObjects.instanceof,
    _props = coreObjects.props,
    this = coreObjects.this,

    -- constructors
    Object = Object,
    Function = Function,
    Array = Array,
    Boolean = Boolean,
    Number = Number,
    String = String,
    Date = Date,
    RegExp = RegExp,
    Error = Error,
    TypeError = TypeError,
    SyntaxError = SyntaxError,
    ReferenceError = ReferenceError,
    RangeError = RangeError,
    URIError = URIError,
    EvalError = EvalError,

    -- others
    console = others.console,
    JSON = json,
    Math = Math,
    _lshift = bit.lshift,
    _rshift = bit.rshift,
    _arshift = bit.arshift,
    _bor = bit.bor,
    _bxor = bit.bxor,
    _band = bit.band,
    _bnot = bit.bnot,

    -- standard library export
    _tostr = tostring,
    _pcall = pcall,
    _ipairs = ipairs,
    _seq = table.remove
}

if jit ~= nil then
    export._wenv = setfenv
end

if _nodejs ~= nil then
    local nodejs = require("castl.nodejs")
    export.require = nodejs.require
end

return export

end
end

end

local IO_Lua = require("IO_Lua")

local _ENV = require("castl.runtime");
local PS;

PS = _obj({});
(function (this, exports)
exports.print = (function (this, str)
do return (function (this)
IO_Lua.print(str);
end); end
end);
end)(_ENV,(function() PS["Language.Lua.IO"] = ((function() local _lev=PS["Language.Lua.IO"]; return _bool(_lev) and _lev or _obj({}) end)()); return PS["Language.Lua.IO"] end)());
(function (this, SPS)
local Sforeign,exports;
SPS["Language.Lua.IO"] = ((function() local _lev=SPS["Language.Lua.IO"]; return _bool(_lev) and _lev or _obj({}) end)());
exports = SPS["Language.Lua.IO"];
Sforeign = SPS["Language.Lua.IO"];
exports["print"] = Sforeign.print;
end)(_ENV,PS);
(function (this, SPS)
local main,Language__Lua__IO,exports;
SPS["Main"] = ((function() local _lev=SPS["Main"]; return _bool(_lev) and _lev or _obj({}) end)());
exports = SPS["Main"];
Language__Lua__IO = SPS["Language.Lua.IO"];
main = Language__Lua__IO:print("Power of PureScript!");
exports["main"] = main;
end)(_ENV,PS);
PS["Main"]:main();
